@@ -187,7 +187,7 @@ struct Node
     unsigned int array_height, this_row;
 #endif
 
-    unsigned vardir : 2;  /* see nodeVarDir, this is an internal value (currently unused) */
+    unsigned vardir : 2;  /* see nodeVarDir, this is an internal value (currently used only by extload and FST) */
     unsigned vartype : 5; /* see nodeVarType, this is an internal value */
 
     unsigned extvals : 1; /* was formerly a pointer to ExtNode "ext", now simply a flag */
@@ -223,6 +223,14 @@ static const char *vartype_strings[] = { \
     "realtime", \
     "port", \
     "string", \
+    "bit", \
+    "logic", \
+    "int", \
+    "s_int", \
+    "l_int", \
+    "byte", \
+    "enum", \
+    "s_real", \
     "net", \
     "alias", \
     "missing" \
@@ -256,18 +264,40 @@ enum nodeVarType {
     ND_VCD_PORT		   = 21,
     ND_GEN_STRING          = 22,    /* generic string type */
 
-    ND_GEN_NET		   = 23,    /* used for AE2 */
-    ND_GEN_ALIAS	   = 24,
-    ND_GEN_MISSING	   = 25,
+    ND_SV_BIT              = 23,    /* SV types */
+    ND_SV_LOGIC            = 24,
+    ND_SV_INT              = 25,
+    ND_SV_SHORTINT         = 26,
+    ND_SV_LONGINT          = 27,
+    ND_SV_BYTE             = 28,
+    ND_SV_ENUM             = 29,
+    ND_SV_SHORTREAL        = 30,
+    
 
-    ND_VARTYPE_MAX	   = 25
+    ND_GEN_NET		   = 31,    /* used for AE2 */
+    ND_GEN_ALIAS	   = 32,
+    ND_GEN_MISSING	   = 33,
+
+    ND_VARTYPE_MAX	   = 33
 };
 
+
+#define WAVE_NODEVARDIR_STR \
+static const char *vardir_strings[] = { \
+    "", \
+    "I", \
+    "O", \
+    "IO" \
+};
+
+
 enum nodeVarDir {
     ND_DIR_IMPLICIT        = 0,
     ND_DIR_IN              = 1,
     ND_DIR_OUT             = 2,
-    ND_DIR_INOUT           = 3
+    ND_DIR_INOUT           = 3,
+
+    ND_DIR_MAX             = 3
 };
 
 typedef struct BitAttributes
