@@ -213,15 +213,15 @@ if(GLOBALS->is_lxt)
 	import_lxt_trace(np);
 	}
 else
+if(GLOBALS->extload) /*needs to be ahead of is_lx2 as now can be is_lx2 with FsdbReader */
+	{
+	import_extload_trace(np);
+	}
+else
 if(GLOBALS->is_lx2)
 	{
 	import_lx2_trace(np);
 	}
-else
-if(GLOBALS->extload)
-	{
-	import_extload_trace(np);
-	}
 else
 	{
 	fprintf(stderr, "Internal error with mvlfac trace handling, exiting.\n");
@@ -329,7 +329,8 @@ while(h[0])	/* should never exit through this point the way we set up histents w
 				case 'w': case 'W':
 					enc = AN_W; break;
 
-				default: enc = enc & AN_MSK; break;
+				default: 
+					enc = enc & AN_MSK; break;
 				}
 			}
 			else
