@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 2009-2012.
+ * Copyright (c) Tony Bybell 2009-2013.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -13,6 +13,8 @@
 #include "vzt.h"
 #include "lx2.h"
 
+#include "fsdb_wrapper_api.h"
+
 #ifndef _MSC_VER
 #include <unistd.h>
 #endif
@@ -48,8 +50,21 @@ fprintf(stderr, "%s", extload_loader_fail_msg);
 exit(255);
 }
 
+void fsdb_import_masked(void)
+{
+fprintf(stderr, "%s", extload_loader_fail_msg);
+exit(255);
+}
+
+void fsdb_set_fac_process_mask(nptr np)
+{
+fprintf(stderr, "%s", extload_loader_fail_msg);
+exit(255);
+}
+
 #else
 
+#ifndef WAVE_FSDB_READER_IS_PRESENT
 static int last_modification_check(void)
 {
 #ifdef HAVE_SYS_STAT_H
@@ -96,29 +111,217 @@ if(GLOBALS->extload_lastmod)
 return(1);
 #endif
 }
+#endif
 
-
-static char *get_varname(unsigned char *vtp, void *xc)
+#ifdef WAVE_FSDB_READER_IS_PRESENT
+static char *get_varname(char *sbuff, unsigned char *vtp, unsigned char *vdp, int i)
+{
+#else
+static char *get_varname(unsigned char *vtp, unsigned char *vdp, int i)
 {
 static char sbuff[65537];
+#endif
 char * rc;
 int vt, vt_len;
 
+#ifndef WAVE_FSDB_READER_IS_PRESENT
 for(;;)
+#endif
 	{
+#ifndef WAVE_FSDB_READER_IS_PRESENT
 	rc = fgets(sbuff, 65536, GLOBALS->extload);
-	if(!rc)
+	if(rc)
+		{
+		if(isspace(rc[0]))
+			{
+			char *snp;
+			char sbuff2[65537];
+
+			sbuff2[0] = 0;
+
+			if((snp=strstr(rc+1, "Struct Name:")))
+				{
+				sscanf(rc+14,"%s", sbuff2);
+				if(sbuff2[0])
+					{
+					sprintf(rc, "Scope: vcd_struct %s NULL\n", sbuff2);
+					}
+				} 
+			else
+			if((snp=strstr(rc+1, "Struct End")))
+				{
+				sprintf(rc, "Upscope:\n");
+				} 
+			}
+		}
+	else
 		{
 		return(NULL);
 		}
+#else
+	rc = sbuff;
+#endif
 
-        if(rc[0] == 'V')
-                {
-                if(!strncmp("Var: ", rc, 5))
+	if((rc[0] == 'V') && (i >= 0))
+		{
+#ifndef WAVE_FSDB_READER_IS_PRESENT
+		if(!strncmp("Var: ", rc, 5))
+#endif
 			{
 			char *pnt = rc + 5;
-			char *vtyp_nam = pnt;
-			char *cpyto = sbuff;
+			char *last_l = NULL;
+			char typ[64];
+			char *esc = NULL;
+			char *lb = NULL;
+			char *colon = NULL;
+			char *rb = NULL;
+			int state = 0;
+			char *vtyp_nam;
+			char *cpyto;
+			char *pntd;
+	
+			sscanf(rc + 5, "%s", typ);
+
+			while(*pnt)
+				{
+				if((pnt[0] == 'l') && (pnt[1] == ':'))
+					{
+					last_l = pnt;
+					}
+				else if(pnt[0] == '\\')
+					{
+					esc = pnt;
+					}
+				else if(!last_l)
+					{
+					if(pnt[0] == '[')
+						{
+						lb = pnt;
+						colon = NULL;
+						rb = NULL;
+						state = 1;
+						}
+					else if(pnt[0] == ']')
+						{
+						rb = pnt;
+						state = 0;
+						if(!isspace(pnt[1]))
+							{
+							lb = colon = rb = NULL;
+							}
+						}
+					else if(pnt[0] == ':')
+						{
+						if(state)
+							{
+							colon = pnt;
+							}
+						}
+					}
+				
+				pnt++;
+				}
+
+			if(last_l)
+				{
+				unsigned int l, r;
+				char s1[32];
+				unsigned int d2;
+				sscanf(last_l+2, "%u r:%u %s %u", &l, &r, s1, &d2);
+
+				GLOBALS->extload_idcodes[i] = d2;
+				if(GLOBALS->extload_inv_idcodes[d2] == 0) GLOBALS->extload_inv_idcodes[d2] = i+1; /* root alias */
+
+				if(!strcmp("vcd_real", typ))
+					{
+					GLOBALS->mvlfacs_vzt_c_3[i].flags = VZT_RD_SYM_F_DOUBLE;
+					GLOBALS->extload_node_block[i].msi=0;				
+					GLOBALS->extload_node_block[i].lsi=0;				
+					GLOBALS->mvlfacs_vzt_c_3[i].len=64;
+					}
+				else
+				if(!strcmp("vcd_integer", typ))
+					{
+					GLOBALS->mvlfacs_vzt_c_3[i].flags = VZT_RD_SYM_F_INTEGER;
+					GLOBALS->extload_node_block[i].msi=0;				
+					GLOBALS->extload_node_block[i].lsi=0;				
+					GLOBALS->mvlfacs_vzt_c_3[i].len=32;
+					}
+				else
+					{
+					int len_parse = 1;
+
+					GLOBALS->mvlfacs_vzt_c_3[i].len=(l>r) ? (l-r+1) : (r-l+1);
+
+					if(esc && lb && rb)
+						{
+						GLOBALS->extload_node_block[i].msi = atoi(lb+1);
+						if(colon)
+							{
+							GLOBALS->extload_node_block[i].lsi = atoi(colon+1);
+							}
+							else
+							{
+							GLOBALS->extload_node_block[i].lsi = GLOBALS->extload_node_block[i].msi;
+							}						
+
+						len_parse = (GLOBALS->extload_node_block[i].msi > GLOBALS->extload_node_block[i].lsi)
+								? (GLOBALS->extload_node_block[i].msi - GLOBALS->extload_node_block[i].lsi + 1)
+								: (GLOBALS->extload_node_block[i].lsi - GLOBALS->extload_node_block[i].msi + 1);
+
+						if(len_parse != GLOBALS->mvlfacs_vzt_c_3[i].len)
+							{
+							GLOBALS->extload_node_block[i].msi=l;				
+							GLOBALS->extload_node_block[i].lsi=r;				
+							}
+						}
+						else
+						{
+						if(lb && !l && !r) /* fix for stranded signals */
+							{
+							GLOBALS->extload_node_block[i].msi=atoi(lb+1);				
+							GLOBALS->extload_node_block[i].lsi=atoi(lb+1);				
+							}
+							else
+							{
+							GLOBALS->extload_node_block[i].msi=l;				
+							GLOBALS->extload_node_block[i].lsi=r;				
+							}
+						}
+
+					GLOBALS->mvlfacs_vzt_c_3[i].flags = VZT_RD_SYM_F_BITS; 
+					}
+				}
+
+			/* now extract directional/type information */
+			pnt = rc + 5;
+			vtyp_nam = pnt;
+			cpyto = sbuff;
+			pntd = strrchr(last_l ? last_l : pnt, ':');
+
+			if(pntd)
+				{
+				unsigned char vd = ND_DIR_IMPLICIT;
+
+				pntd = strchr(pntd, ' ');
+				if(pntd)
+					{
+					pntd++;
+					if(*pntd == 'o')
+						{
+						vd = ND_DIR_OUT;
+						GLOBALS->nonimplicit_direction_encountered = 1;
+						}
+					else
+					if(!strncmp(pntd, "in", 2))
+						{
+						vd = (pntd[2] == 'p') ? ND_DIR_IN : ND_DIR_INOUT;
+						GLOBALS->nonimplicit_direction_encountered = 1;
+						}
+					}
+
+				if(vdp) { *vdp = vd; }
+				}
 
 			while(*pnt)
 				{
@@ -178,7 +381,33 @@ for(;;)
 					/* if((*pnt == '[')||(isspace(*pnt))) break; */
 
 					if(isspace(*pnt)) break;
-					if((*pnt == '[') && (pnt == strrchr(pnt, '['))) break; /* fixes arrays */
+					if((*pnt == '[') && (pnt == strrchr(pnt, '['))) /* fix for arrays */
+						{
+						/* now to fix possible generate... */
+						char *pnt2 = pnt;
+						char lastch = *pnt2;
+						int colon_seen = 0;
+
+						pnt2++;
+						while(*pnt2 && !isspace(*pnt2) && (*pnt2 != '[')) 
+							{ 
+							lastch = *pnt2; pnt2++; 
+							if(lastch == ':') { colon_seen = 1; }
+							};
+
+						if(lastch == ']') /* fix for NC verilog arrays */
+							{
+							int rng;
+
+							if(colon_seen) break;
+
+							rng = GLOBALS->extload_node_block[i].msi - GLOBALS->extload_node_block[i].lsi;
+							if(!rng)
+								{
+								break;
+								}
+							}
+						}
 
 					if(*pnt == '\\') /* this is not strictly correct, but fixes generic ranges from icarus */
 						{
@@ -195,7 +424,9 @@ for(;;)
 	else
         if(rc[0] == 'S')
                 {
+#ifndef WAVE_FSDB_READER_IS_PRESENT
 		if(!strncmp(rc, "Scope:", 6))
+#endif
 			{
 			char vht[2048];
 			char cname[2048];
@@ -204,7 +435,7 @@ for(;;)
 
 			vht[0] = vht[4] = vht[5] = cname[0] = ctype[0] = 0;
 			sscanf(rc+6, "%s %s %s", vht, cname, ctype);
-                        GLOBALS->fst_scope_name = fstReaderPushScope(xc, cname, GLOBALS->mod_tree_parent);
+                        GLOBALS->fst_scope_name = fstReaderPushScope(GLOBALS->extload_xc, cname, GLOBALS->mod_tree_parent);
 			if(!strcmp(ctype, "NULL")) { ctype[0] = 0; }
 
 			if(!strncmp(vht, "vcd_", 4))
@@ -215,6 +446,17 @@ for(;;)
 		                        case 't':       ttype = TREE_VCD_ST_TASK; break;
 		                        case 'f':	ttype = (vht[5] == 'u') ? TREE_VCD_ST_FUNCTION : TREE_VCD_ST_FORK; break;
 		                        case 'b':       ttype = TREE_VCD_ST_BEGIN; break;
+		                        case 'g':       ttype = TREE_VCD_ST_GENERATE; break;
+					case 's':	ttype = TREE_VCD_ST_STRUCT; break;
+		                        default:        ttype = TREE_UNKNOWN; break;
+		                        }
+				}
+			else
+			if(!strncmp(vht, "sv_", 3))
+				{
+		                switch(vht[3])
+					{
+		                        case 'i':       ttype = TREE_VCD_ST_INTERFACE; break;
 		                        default:        ttype = TREE_UNKNOWN; break;
 		                        }
 				}
@@ -229,327 +471,102 @@ for(;;)
 	else
         if(rc[0] == 'U')
                 {
-                GLOBALS->mod_tree_parent = fstReaderGetCurrentScopeUserInfo(xc);
-                GLOBALS->fst_scope_name = fstReaderPopScope(xc);
+                GLOBALS->mod_tree_parent = fstReaderGetCurrentScopeUserInfo(GLOBALS->extload_xc);
+                GLOBALS->fst_scope_name = fstReaderPopScope(GLOBALS->extload_xc);
 		}
 	}
+
+return(NULL);
 }
 
-/*
- * mainline
- */
-TimeType extload_main(char *fname, char *skip_start, char *skip_end)
+
+#ifdef WAVE_FSDB_READER_IS_PRESENT
+static void process_extload_variable(char *s_gv)
+#else
+static void process_extload_variable(void)
+#endif
 {
-char sbuff[65537];
-int max_idcode;
-unsigned int msk = 0;
-unsigned char vt_prev, vt, nvt;
-void *xc = NULL;
-
 int i;
+unsigned char vt, nvt;
+unsigned char vd;
 struct Node *n;
-struct symbol *s, *prevsymroot=NULL, *prevsym=NULL;
-struct symbol *sym_block = NULL;
-struct Node *node_block = NULL;
-char **namecache = NULL;
+struct symbol *s;
+char buf[65537];
+char *str;	
+struct fac *f;
+char *fnam;
+int flen;
 
-if(!(GLOBALS->extload=fopen(fname, "rb")))
+i = GLOBALS->extload_i;
+
+if(i<0)
 	{
-	GLOBALS->extload_already_errored = 1;
-	return(LLDescriptor(0)); 	/* look at GLOBALS->vzt_vzt_c_1 in caller for success status... */
-	}
-fclose(GLOBALS->extload);
+#ifdef WAVE_FSDB_READER_IS_PRESENT
+	fnam = get_varname(s_gv, &GLOBALS->extload_vt_prev, &GLOBALS->extload_vd_prev, 0);
+	flen = strlen(fnam);
 
-/* SPLASH */                            splash_create();
-
-last_modification_check();
-sprintf(sbuff, "%s -info %s 2>&1", EXTLOAD_PATH, fname);
-GLOBALS->extload = popen(sbuff, "r");
-for(;;)
-	{
-	char * rc = fgets(sbuff, 65536, GLOBALS->extload);
-	if(!rc) break;
-
-	switch(rc[0])
+	if(GLOBALS->extload_hlen)
 		{
-		case 's':
-			if(!strncmp("scale unit         :", rc, 20))
-				{
-				char *pnt = rc+20;
-
-				GLOBALS->time_scale = atoi(rc+20);
-				GLOBALS->time_dimension = 'n';
-				while(*pnt)
-					{
-					if(isalpha(*pnt))
-						{
-						GLOBALS->time_dimension = tolower(*pnt);
-						break;
-						}
-					pnt++;
-					}
-				
-				msk |= 1;
-				}
-			break;
-
-		case 'm':
-			if(!strncmp("minimum xtag       : (", rc, 22))
-				{
-				unsigned int lo = 0, hi = 0;
-				sscanf(rc + 22, "%u %u", &hi, &lo);
-				GLOBALS->min_time = (TimeType)((((UTimeType)hi)<<32) + ((UTimeType)lo));
-
-				msk |= 2;
-				}
-			else
-			if(!strncmp("maximum xtag       : (", rc, 22))
-				{
-				unsigned int lo = 0, hi = 0;
-				sscanf(rc + 22, "%u %u", &hi, &lo);
-				GLOBALS->max_time = (TimeType)((((UTimeType)hi)<<32) + ((UTimeType)lo));
-
-				msk |= 4;
-				}
-			else
-			if(!strncmp("max var idcode     :", rc, 20))
-				{
-				sscanf(rc + 21, "%d", &max_idcode);
-
-				msk |= 8;
-				}
-
-			break;
-
-		case 'v':
-			if(!strncmp("var creation cnt   :", rc, 20))
-				{
-				sscanf(rc + 21, "%d", &GLOBALS->numfacs);
-
-				msk |= 16;
-				}
-
-		case 'f':
-			if(!strncmp("file status        : finished", rc, 29))
-				{
-				msk |= 32;
-				}
-			break;
-
-		default:
-			break;
+		GLOBALS->extload_namecache[0]=malloc_2(GLOBALS->extload_hlen+1+flen+1);
+		strcpy(GLOBALS->extload_namecache[0], GLOBALS->fst_scope_name);
+		*(GLOBALS->extload_namecache[0]+GLOBALS->extload_hlen) = '.';
+		strcpy(GLOBALS->extload_namecache[0]+GLOBALS->extload_hlen+1, fnam);
 		}
-	}
-pclose(GLOBALS->extload);
-
-if(msk != (1+2+4+8+16+32))
-	{
-	fprintf(stderr, EXTLOAD"Could not initialize '%s' properly.\n", fname);
-	if((msk & (1+2+4+8+16+32)) == (1+2+4+8+16))
-		{
-		fprintf(stderr, EXTLOAD"File is not finished dumping.\n");
-		}
-	GLOBALS->extload_already_errored = 1;
-	return(LLDescriptor(0));
-	}
-
-GLOBALS->min_time *= GLOBALS->time_scale;
-GLOBALS->max_time *= GLOBALS->time_scale;
-
-GLOBALS->mvlfacs_vzt_c_3=(struct fac *)calloc_2(GLOBALS->numfacs,sizeof(struct fac));
-GLOBALS->vzt_table_vzt_c_1=(struct lx2_entry *)calloc_2(GLOBALS->numfacs, sizeof(struct lx2_entry));
-namecache=(char **)calloc_2(GLOBALS->numfacs, sizeof(char *));
-sym_block = (struct symbol *)calloc_2(GLOBALS->numfacs, sizeof(struct symbol));
-node_block=(struct Node *)calloc_2(GLOBALS->numfacs,sizeof(struct Node));
-GLOBALS->extload_idcodes=(unsigned int *)calloc_2(GLOBALS->numfacs, sizeof(unsigned int));
-GLOBALS->extload_inv_idcodes=(int *)calloc_2(max_idcode+1, sizeof(int));
-
-if(!last_modification_check()) { GLOBALS->extload_already_errored = 1; return(LLDescriptor(0)); }
-sprintf(sbuff, "%s -hier_tree %s 2>&1", EXTLOAD_PATH, fname);
-GLOBALS->extload = popen(sbuff, "r");
-i = 0;
-for(;;)
-	{
-	char * rc = fgets(sbuff, 65536, GLOBALS->extload);
-	if(!rc) break;
-
-	if(rc[0] == 'V')
-		{
-		if(!strncmp("Var:", rc, 4))
-			{
-			char *pnt = rc + 5;
-			char *last_l = NULL;
-			char typ[64];
-			char *esc = NULL;
-			char *lb = NULL;
-			char *colon = NULL;
-			char *rb = NULL;
-			int state = 0;
-
-			sscanf(rc + 5, "%s", typ);
-
-			while(*pnt)
-				{
-				if((pnt[0] == 'l') && (pnt[1] == ':'))
-					{
-					last_l = pnt;
-					}
-				else if(pnt[0] == '\\')
-					{
-					esc = pnt;
-					}
-				else if(pnt[0] == '[')
-					{
-					lb = pnt;
-					colon = NULL;
-					state = 1;
-					}
-				else if(pnt[0] == ']')
-					{
-					rb = pnt;
-					state = 0;
-					}
-				else if(pnt[0] == ':')
-					{
-					if(state)
-						{
-						colon = pnt;
-						}
-					}
-				
-				pnt++;
-				}
-
-			if(last_l)
-				{
-				unsigned int l, r;
-				char s1[32], s3[32], s4[32];
-				unsigned int d2;
-				sscanf(last_l, "l:%u r:%u %s %u %s %s", &l, &r, s1, &d2, s3, s4);
-
-				GLOBALS->extload_idcodes[i] = d2;
-				if(GLOBALS->extload_inv_idcodes[d2] == 0) GLOBALS->extload_inv_idcodes[d2] = i+1; /* root alias */
-
-				if(!strcmp("vcd_real", typ))
-					{
-					GLOBALS->mvlfacs_vzt_c_3[i].flags = VZT_RD_SYM_F_DOUBLE;
-					node_block[i].msi=0;				
-					node_block[i].lsi=0;				
-					GLOBALS->mvlfacs_vzt_c_3[i].len=64;
-					}
-				else
-				if(!strcmp("vcd_integer", typ))
-					{
-					GLOBALS->mvlfacs_vzt_c_3[i].flags = VZT_RD_SYM_F_INTEGER;
-					node_block[i].msi=0;				
-					node_block[i].lsi=0;				
-					GLOBALS->mvlfacs_vzt_c_3[i].len=32;
-					}
-				else
-					{
-					int len_parse = 1;
-
-					GLOBALS->mvlfacs_vzt_c_3[i].len=(l>r) ? (l-r+1) : (r-l+1);
-
-					if(esc && lb && rb)
-						{
-						node_block[i].msi = atoi(lb+1);
-						if(colon)
-							{
-							node_block[i].lsi = atoi(colon+1);
-							}
-							else
-							{
-							node_block[i].lsi = node_block[i].msi;
-							}						
-
-						len_parse = (node_block[i].msi > node_block[i].lsi)
-								? (node_block[i].msi - node_block[i].lsi + 1)
-								: (node_block[i].lsi - node_block[i].msi + 1);
-
-						if(len_parse != GLOBALS->mvlfacs_vzt_c_3[i].len)
-							{
-							node_block[i].msi=l;				
-							node_block[i].lsi=r;				
-							}
-						}
-						else
-						{
-						node_block[i].msi=l;				
-						node_block[i].lsi=r;				
-						}
-
-					GLOBALS->mvlfacs_vzt_c_3[i].flags = VZT_RD_SYM_F_BITS; 
-					}
-				}
-
-			i++;
-			}
-		}
-
-	}
-
-pclose(GLOBALS->extload);
-
-if(i==GLOBALS->numfacs)
-	{
-	fprintf(stderr, EXTLOAD"Finished building %d facs.\n", GLOBALS->numfacs);
-	}
 	else
-	{
-	fprintf(stderr, EXTLOAD"Fac count mismatch: %d expected vs %d found, exiting.\n", GLOBALS->numfacs, i);
-	GLOBALS->extload_already_errored = 1;
-	return(LLDescriptor(0));
+		{
+		GLOBALS->extload_namecache[0]=malloc_2(flen+1);
+		strcpy(GLOBALS->extload_namecache[0], fnam);
+		}
+#else
+	fnam = get_varname(&GLOBALS->extload_vt_prev, &GLOBALS->extload_vd_prev, 0);
+	flen = strlen(fnam);
+
+	GLOBALS->extload_namecache[0]=malloc_2(flen+1);
+	strcpy(GLOBALS->extload_namecache[0], fnam);
+#endif
 	}
-/* SPLASH */                            splash_sync(1, 5);
-
-if(!last_modification_check()) { GLOBALS->extload_already_errored = 1; return(LLDescriptor(0)); }
-sprintf(sbuff, "%s -hier_tree %s 2>&1", EXTLOAD_PATH, fname);
-GLOBALS->extload = popen(sbuff, "r");
-
-/* do your stuff here..all useful info has been initialized by now */
-
-if(!GLOBALS->hier_was_explicitly_set)    /* set default hierarchy split char */
-        {
-        GLOBALS->hier_delimeter='.';
-        }
-
-xc = fstReaderOpenForUtilitiesOnly();
-
-if(GLOBALS->numfacs)
+else
 	{
-	char *fnam = get_varname(&vt_prev, xc);
-	int flen = strlen(fnam);
-	namecache[0]=malloc_2(flen+1);
-	strcpy(namecache[0], fnam);
-	}
-
-for(i=0;i<GLOBALS->numfacs;i++)
-        {
-	char buf[65537];
-	char *str;	
-	struct fac *f;
-
-	vt = vt_prev;
+	vt = GLOBALS->extload_vt_prev;
+	vd = GLOBALS->extload_vd_prev;
 	if(i!=(GLOBALS->numfacs-1))
 		{
-		char *fnam = get_varname(&vt_prev, xc);
-		int flen = strlen(fnam);
-		namecache[i+1]=malloc_2(flen+1);
-		strcpy(namecache[i+1], fnam);
+#ifdef WAVE_FSDB_READER_IS_PRESENT
+		fnam = get_varname(s_gv, &GLOBALS->extload_vt_prev, &GLOBALS->extload_vd_prev, i+1);
+		flen = strlen(fnam);
+
+		if(GLOBALS->extload_hlen)
+			{
+			GLOBALS->extload_namecache[i+1]=malloc_2(GLOBALS->extload_hlen+1+flen+1);
+			strcpy(GLOBALS->extload_namecache[i+1], GLOBALS->fst_scope_name);
+			*(GLOBALS->extload_namecache[i+1]+GLOBALS->extload_hlen) = '.';
+			strcpy(GLOBALS->extload_namecache[i+1]+GLOBALS->extload_hlen+1, fnam);
+			}
+		else
+			{
+			GLOBALS->extload_namecache[i+1]=malloc_2(flen+1);
+			strcpy(GLOBALS->extload_namecache[i+1], fnam);
+			}
+#else
+		fnam = get_varname(&GLOBALS->extload_vt_prev, &GLOBALS->extload_vd_prev, i+1);
+		flen = strlen(fnam);
+
+		GLOBALS->extload_namecache[i+1]=malloc_2(flen+1);
+		strcpy(GLOBALS->extload_namecache[i+1], fnam);
+#endif
 		}
 
 	if(i>1)
 		{
-		free_2(namecache[i-2]);
-		namecache[i-2] = NULL;
+		free_2(GLOBALS->extload_namecache[i-2]);
+		GLOBALS->extload_namecache[i-2] = NULL;
 		}
 
 	f=GLOBALS->mvlfacs_vzt_c_3+i;
 
 	if((f->len>1)&& (!(f->flags&(VZT_RD_SYM_F_INTEGER|VZT_RD_SYM_F_DOUBLE|VZT_RD_SYM_F_STRING))) )
 		{
-		int len=sprintf(buf, "%s[%d:%d]", namecache[i],node_block[i].msi, node_block[i].lsi);
+		int len=sprintf(buf, "%s[%d:%d]", GLOBALS->extload_namecache[i],GLOBALS->extload_node_block[i].msi, GLOBALS->extload_node_block[i].lsi);
 		str=malloc_2(len+1);
 
 		if(!GLOBALS->alt_hier_delimeter)
@@ -560,19 +577,19 @@ for(i=0;i<GLOBALS->numfacs;i++)
 			{
 			strcpy_vcdalt(str, buf, GLOBALS->alt_hier_delimeter);
 			}
-		s=&sym_block[i];
+		s=&GLOBALS->extload_sym_block[i];
 	        symadd_name_exists_sym_exists(s,str,0);
-		prevsymroot = prevsym = NULL;
+		GLOBALS->extload_prevsymroot = GLOBALS->extload_prevsym = NULL;
 		}
 	else if ( 
 			((f->len==1)&&(!(f->flags&(VZT_RD_SYM_F_INTEGER|VZT_RD_SYM_F_DOUBLE|VZT_RD_SYM_F_STRING)))&&
-			((i!=GLOBALS->numfacs-1)&&(!strcmp(namecache[i], namecache[i+1]))))
+			((i!=GLOBALS->numfacs-1)&&(!strcmp(GLOBALS->extload_namecache[i], GLOBALS->extload_namecache[i+1]))))
 			||
-			(((i!=0)&&(!strcmp(namecache[i], namecache[i-1]))) &&
-			(node_block[i].msi!=-1)&&(node_block[i].lsi!=-1))
+			(((i!=0)&&(!strcmp(GLOBALS->extload_namecache[i], GLOBALS->extload_namecache[i-1]))) &&
+			(GLOBALS->extload_node_block[i].msi!=-1)&&(GLOBALS->extload_node_block[i].lsi!=-1))
 		)
 		{
-		int len = sprintf(buf, "%s[%d]", namecache[i],node_block[i].msi);
+		int len = sprintf(buf, "%s[%d]", GLOBALS->extload_namecache[i],GLOBALS->extload_node_block[i].msi);
 		str=malloc_2(len+1);
 		if(!GLOBALS->alt_hier_delimeter)
 			{
@@ -582,44 +599,44 @@ for(i=0;i<GLOBALS->numfacs;i++)
 			{
 			strcpy_vcdalt(str, buf, GLOBALS->alt_hier_delimeter);
 			}
-		s=&sym_block[i];
+		s=&GLOBALS->extload_sym_block[i];
 	        symadd_name_exists_sym_exists(s,str,0);
-		if((prevsym)&&(i>0)&&(!strcmp(namecache[i], namecache[i-1])))	/* allow chaining for search functions.. */
+		if((GLOBALS->extload_prevsym)&&(i>0)&&(!strcmp(GLOBALS->extload_namecache[i], GLOBALS->extload_namecache[i-1])))	/* allow chaining for search functions.. */
 			{
-			prevsym->vec_root = prevsymroot;
-			prevsym->vec_chain = s;
-			s->vec_root = prevsymroot;
-			prevsym = s;
+			GLOBALS->extload_prevsym->vec_root = GLOBALS->extload_prevsymroot;
+			GLOBALS->extload_prevsym->vec_chain = s;
+			s->vec_root = GLOBALS->extload_prevsymroot;
+			GLOBALS->extload_prevsym = s;
 			}
 			else
 			{
-			prevsymroot = prevsym = s;
+			GLOBALS->extload_prevsymroot = GLOBALS->extload_prevsym = s;
 			}
 		}
 		else
 		{
-		str=malloc_2(strlen(namecache[i])+1);
+		str=malloc_2(strlen(GLOBALS->extload_namecache[i])+1);
 		if(!GLOBALS->alt_hier_delimeter)
 			{
-			strcpy(str, namecache[i]);
+			strcpy(str, GLOBALS->extload_namecache[i]);
 			}
 			else
 			{
-			strcpy_vcdalt(str, namecache[i], GLOBALS->alt_hier_delimeter);
+			strcpy_vcdalt(str, GLOBALS->extload_namecache[i], GLOBALS->alt_hier_delimeter);
 			}
-		s=&sym_block[i];
+		s=&GLOBALS->extload_sym_block[i];
 	        symadd_name_exists_sym_exists(s,str,0);
-		prevsymroot = prevsym = NULL;
+		GLOBALS->extload_prevsymroot = GLOBALS->extload_prevsym = NULL;
 
 		if(f->flags&VZT_RD_SYM_F_INTEGER)
 			{
-			node_block[i].msi=31;
-			node_block[i].lsi=0;
+			GLOBALS->extload_node_block[i].msi=31;
+			GLOBALS->extload_node_block[i].lsi=0;
 			GLOBALS->mvlfacs_vzt_c_3[i].len=32;
 			}
 		}
 		
-        n=&node_block[i];
+        n=&GLOBALS->extload_node_block[i];
         n->nname=s->name;
         n->mv.mvlfac = GLOBALS->mvlfacs_vzt_c_3+i;
 	GLOBALS->mvlfacs_vzt_c_3[i].working_node = n;
@@ -657,25 +674,338 @@ for(i=0;i<GLOBALS->numfacs;i++)
 	        default:                nvt = ND_UNSPECIFIED_DEFAULT; break;
 	        }
 	n->vartype = nvt;
+	n->vardir = vd;
         }
 
-while(get_varname(&vt_prev, xc)); /* read through end to process all upscopes */
+GLOBALS->extload_i++;
+}
+
+
+#ifdef WAVE_FSDB_READER_IS_PRESENT
+static void extload_hiertree_callback(void *pnt)
+{
+char *s = (char *)pnt;
+
+switch(s[0])
+	{
+	case 'S':
+	case 'U': 	get_varname(s, NULL, NULL, -1);
+			GLOBALS->extload_hlen = GLOBALS->fst_scope_name ? strlen(GLOBALS->fst_scope_name) : 0;
+			break;
+
+	case 'V':	process_extload_variable(s);
+	default:	break;
+	}
+}
+#endif
+
+
+/*
+ * mainline
+ */
+static TimeType extload_main_2(char *fname, char *skip_start, char *skip_end)
+{
+int max_idcode;
+#ifndef WAVE_FSDB_READER_IS_PRESENT
+char sbuff[65537];
+unsigned int msk = 0;
+#endif
+
+int i;
+
+if(!(GLOBALS->extload=fopen(fname, "rb")))
+	{
+	GLOBALS->extload_already_errored = 1;
+	return(LLDescriptor(0)); 	/* look at GLOBALS->vzt_vzt_c_1 in caller for success status... */
+	}
+fclose(GLOBALS->extload);
+
+
+/* SPLASH */                            splash_create();
+
+#ifdef WAVE_FSDB_READER_IS_PRESENT
+GLOBALS->extload_ffr_ctx = fsdbReaderOpenFile(GLOBALS->loaded_file_name);
+GLOBALS->is_lx2 = LXT2_IS_FSDB;
+
+if(GLOBALS->extload_ffr_ctx)
+	{
+	int rv;
+	int mult;
+	char scale;
+	uint64_t tim;
+	struct fsdbReaderGetStatistics_t *gs;
+	int success_count = 0;
+	int attempt_count = 0;
+
+	attempt_count++;
+	rv = fsdbReaderExtractScaleUnit(GLOBALS->extload_ffr_ctx, &mult, &scale);
+	if(rv)
+		{
+		GLOBALS->time_scale = mult;
+		GLOBALS->time_dimension = tolower(scale);
+		success_count++;
+		}
+
+	attempt_count++;
+	rv = fsdbReaderGetMinFsdbTag64(GLOBALS->extload_ffr_ctx, &tim);
+	if(rv)
+		{
+		GLOBALS->min_time = tim;
+		success_count++;
+		}
+	
+	attempt_count++;
+	rv = fsdbReaderGetMaxFsdbTag64(GLOBALS->extload_ffr_ctx, &tim);
+	if(rv)
+		{
+		GLOBALS->max_time = tim;
+		if(GLOBALS->max_time == LLDescriptor(0))
+			{
+			GLOBALS->max_time = LLDescriptor(1);
+			}
+		success_count++;
+		}
+
+	attempt_count++;
+	max_idcode = fsdbReaderGetMaxVarIdcode(GLOBALS->extload_ffr_ctx);
+	if(max_idcode)
+		{
+		success_count++;
+		}
+	
+	attempt_count++;
+	gs = fsdbReaderGetStatistics(GLOBALS->extload_ffr_ctx);
+	if(gs)
+		{
+		GLOBALS->numfacs = gs->varCount;
+		free(gs);
+		success_count++;
+		}
+
+	if(attempt_count != success_count)
+		{
+		fprintf(stderr, EXTLOAD"Could not initialize '%s' properly.\n", fname);
+		GLOBALS->extload_already_errored = 1;
+		return(LLDescriptor(0));
+		}
+	}
+	else
+	{
+	fprintf(stderr, EXTLOAD"Could not initialize '%s' properly.\n", fname);
+	GLOBALS->extload_already_errored = 1;
+	return(LLDescriptor(0));
+	}
+
+#else
+
+last_modification_check();
+sprintf(sbuff, "%s -info %s 2>&1", EXTLOAD_PATH, fname);
+GLOBALS->extload = popen(sbuff, "r");
+for(;;)
+	{
+	char * rc = fgets(sbuff, 65536, GLOBALS->extload);
+	if(!rc) break;
+
+	switch(rc[0])
+		{
+		case 's':
+			if(!strncmp("scale unit", rc, 10))
+				{
+				char *pnt = strchr(rc+10, ':');
+
+				if(pnt)
+					{
+					pnt++;
+					GLOBALS->time_scale = atoi(pnt);
+					GLOBALS->time_dimension = 'n';
+					while(*pnt)
+						{
+						if(isalpha(*pnt))
+							{
+							GLOBALS->time_dimension = tolower(*pnt);
+							break;
+							}
+						pnt++;
+						}
+					
+					msk |= 1;
+					}
+				}
+			break;
+
+		case 'm':
+			if(!strncmp("minimum xtag", rc, 12))
+				{
+				char *pnt = strchr(rc+12, '(');
+				if(pnt)
+					{
+					unsigned int lo = 0, hi = 0;
+					pnt++;
+					sscanf(pnt, "%u %u", &hi, &lo);
+					GLOBALS->min_time = (TimeType)((((UTimeType)hi)<<32) + ((UTimeType)lo));
+
+					msk |= 2;
+					}
+				}
+			else
+			if(!strncmp("maximum xtag", rc, 12))
+				{
+				char *pnt = strchr(rc+12, '(');
+				if(pnt)
+					{
+					unsigned int lo = 0, hi = 0;
+					pnt++;
+					sscanf(pnt, "%u %u", &hi, &lo);
+					GLOBALS->max_time = (TimeType)((((UTimeType)hi)<<32) + ((UTimeType)lo));
+					if(GLOBALS->max_time == LLDescriptor(0))
+						{
+						GLOBALS->max_time = LLDescriptor(1);
+						}
+
+					msk |= 4;
+					}
+				}
+			else
+			if(!strncmp("max var idcode", rc, 14))
+				{
+				char *pnt = strchr(rc+14, ':');
+				if(pnt)
+					{
+					pnt++;
+					sscanf(pnt, "%d", &max_idcode);
+
+					msk |= 8;
+					}
+				}
+
+			break;
+
+		case 'v':
+			if(!strncmp("var creation cnt", rc, 16))
+				{
+				char *pnt = strchr(rc+16, ':');
+				if(pnt)
+					{
+					pnt++;
+					sscanf(pnt, "%d", &GLOBALS->numfacs);
+
+					msk |= 16;
+					}
+				}
+
+		case 'f':
+			if(!strncmp("file status", rc, 11))
+				{
+				char *pnt = strchr(rc+11, ':');
+				if(pnt)
+					{
+					pnt++;
+					if(strstr(pnt, "finished"))
+						{
+						msk |= 32;
+						}
+					}
+				}
+			break;
+
+		default:
+			break;
+		}
+	}
+pclose(GLOBALS->extload);
+
+if(msk != (1+2+4+8+16+32))
+	{
+	fprintf(stderr, EXTLOAD"Could not initialize '%s' properly.\n", fname);
+	if((msk & (1+2+4+8+16+32)) == (1+2+4+8+16))
+		{
+		fprintf(stderr, EXTLOAD"File is not finished dumping.\n");
+		}
+	GLOBALS->extload_already_errored = 1;
+	return(LLDescriptor(0));
+	}
+
+#endif
+
+GLOBALS->min_time *= GLOBALS->time_scale;
+GLOBALS->max_time *= GLOBALS->time_scale;
+
+GLOBALS->mvlfacs_vzt_c_3=(struct fac *)calloc_2(GLOBALS->numfacs,sizeof(struct fac));
+GLOBALS->vzt_table_vzt_c_1=(struct lx2_entry *)calloc_2(GLOBALS->numfacs, sizeof(struct lx2_entry));
+GLOBALS->extload_namecache=(char **)calloc_2(GLOBALS->numfacs, sizeof(char *));
+GLOBALS->extload_sym_block = (struct symbol *)calloc_2(GLOBALS->numfacs, sizeof(struct symbol));
+GLOBALS->extload_node_block=(struct Node *)calloc_2(GLOBALS->numfacs,sizeof(struct Node));
+GLOBALS->extload_idcodes=(unsigned int *)calloc_2(GLOBALS->numfacs, sizeof(unsigned int));
+GLOBALS->extload_inv_idcodes=(int *)calloc_2(max_idcode+1, sizeof(int));
+
+/* SPLASH */                            splash_sync(1, 5);
+
+#ifdef WAVE_FSDB_READER_IS_PRESENT
+
+if(!GLOBALS->hier_was_explicitly_set)    /* set default hierarchy split char */
+        {
+        GLOBALS->hier_delimeter='.';
+        }
+
+GLOBALS->extload_xc = fstReaderOpenForUtilitiesOnly();
+
+GLOBALS->extload_i=-1;
+fsdbReaderReadScopeVarTree(GLOBALS->extload_ffr_ctx, extload_hiertree_callback);
+process_extload_variable(NULL); /* flush out final cached variable */
 
 decorated_module_cleanup(); /* ...also now in gtk2_treesearch.c */
 iter_through_comp_name_table();
 
 for(i=0;((i<2)&&(i<GLOBALS->numfacs));i++)
 	{
-	if(namecache[i])
+	if(GLOBALS->extload_namecache[i])
 		{
-		free_2(namecache[i]);
-		namecache[i] = NULL;
+		free_2(GLOBALS->extload_namecache[i]);
+		GLOBALS->extload_namecache[i] = NULL;
 		}
 	}
-free_2(namecache); namecache = NULL;
+free_2(GLOBALS->extload_namecache); GLOBALS->extload_namecache = NULL;
+
+fstReaderClose(GLOBALS->extload_xc); /* corresponds to fstReaderOpenForUtilitiesOnly() */
+
+#else
+
+if(!last_modification_check()) { GLOBALS->extload_already_errored = 1; return(LLDescriptor(0)); }
+sprintf(sbuff, "%s -hier_tree %s 2>&1", EXTLOAD_PATH, fname);
+GLOBALS->extload = popen(sbuff, "r");
+
+/* do your stuff here..all useful info has been initialized by now */
+
+if(!GLOBALS->hier_was_explicitly_set)    /* set default hierarchy split char */
+        {
+        GLOBALS->hier_delimeter='.';
+        }
+
+GLOBALS->extload_xc = fstReaderOpenForUtilitiesOnly();
+
+for(GLOBALS->extload_i=-1;(GLOBALS->numfacs) && (GLOBALS->extload_i<GLOBALS->numfacs);)
+	{
+	process_extload_variable();
+	}
+while(get_varname(&GLOBALS->extload_vt_prev, NULL, -1)); /* read through end to process all upscopes */
+
+decorated_module_cleanup(); /* ...also now in gtk2_treesearch.c */
+iter_through_comp_name_table();
+
+for(i=0;((i<2)&&(i<GLOBALS->numfacs));i++)
+	{
+	if(GLOBALS->extload_namecache[i])
+		{
+		free_2(GLOBALS->extload_namecache[i]);
+		GLOBALS->extload_namecache[i] = NULL;
+		}
+	}
+free_2(GLOBALS->extload_namecache); GLOBALS->extload_namecache = NULL;
 pclose(GLOBALS->extload);
 
-fstReaderClose(xc); /* corresponds to fstReaderOpenForUtilitiesOnly() */
+fstReaderClose(GLOBALS->extload_xc); /* corresponds to fstReaderOpenForUtilitiesOnly() */
+
+#endif
 
 /* SPLASH */                            splash_sync(2, 5);  
 GLOBALS->facs=(struct symbol **)malloc_2(GLOBALS->numfacs*sizeof(struct symbol *));
@@ -685,7 +1015,7 @@ if(GLOBALS->fast_tree_sort)
         for(i=0;i<GLOBALS->numfacs;i++)
                 {
                 int len;
-                GLOBALS->facs[i]=&sym_block[i]; 
+                GLOBALS->facs[i]=&GLOBALS->extload_sym_block[i]; 
                 if((len=strlen(GLOBALS->facs[i]->name))>GLOBALS->longestname) GLOBALS->longestname=len;
                 }
                                 
@@ -717,7 +1047,7 @@ if(GLOBALS->fast_tree_sort)
 #endif
 		int len;
 
-		GLOBALS->facs[i]=&sym_block[i];
+		GLOBALS->facs[i]=&GLOBALS->extload_sym_block[i];
 		subst=GLOBALS->facs[i]->name;
 	        if((len=strlen(subst))>GLOBALS->longestname) GLOBALS->longestname=len;
 #ifdef WAVE_HIERFIX
@@ -792,6 +1122,25 @@ return(GLOBALS->max_time);
 }
 
 
+TimeType extload_main(char *fname, char *skip_start, char *skip_end)
+{
+TimeType tt = extload_main_2(fname, skip_start, skip_end);
+
+if(!tt)
+	{
+        if(GLOBALS->extload_ffr_ctx)
+                {
+#ifdef WAVE_FSDB_READER_IS_PRESENT
+                fsdbReaderClose(GLOBALS->extload_ffr_ctx);
+#endif
+                GLOBALS->extload_ffr_ctx = NULL;
+                }
+	}
+
+return(tt);
+}
+
+
 /*
  * extload callback (only does bits for now)
  */
@@ -913,11 +1262,52 @@ if(GLOBALS->extload_inv_idcodes[txidx_in_trace] < 0)
 
 GLOBALS->extload_inv_idcodes[txidx_in_trace] = - (txidx + 1); 
 
+#ifndef WAVE_FSDB_READER_IS_PRESENT
 fprintf(stderr, EXTLOAD"Import: %s\n", np->nname);
+#endif
 
 /* new stuff */
 len = np->mv.mvlfac->len;
 
+#ifdef WAVE_FSDB_READER_IS_PRESENT
+{
+void *hdl;
+
+/* fsdbReaderAddToSignalList(GLOBALS->extload_ffr_ctx, txidx_in_trace); */
+/* fsdbReaderLoadSignals(GLOBALS->extload_ffr_ctx); */
+
+hdl = fsdbReaderCreateVCTraverseHandle(GLOBALS->extload_ffr_ctx, txidx_in_trace);
+if(fsdbReaderHasIncoreVC(GLOBALS->extload_ffr_ctx, hdl))
+	{
+	TimeType mxt = (TimeType)fsdbReaderGetMinXTag(GLOBALS->extload_ffr_ctx, hdl);
+	fsdbReaderGotoXTag(GLOBALS->extload_ffr_ctx, hdl, mxt);
+
+	for(;;)
+		{
+		void *val_ptr;
+		char *b;
+		if(!fsdbReaderGetVC(GLOBALS->extload_ffr_ctx, hdl, &val_ptr))
+			{
+			break;
+			}
+
+		b = fsdbReaderTranslateVC(hdl, val_ptr);
+		extload_callback(&mxt, &txidx, &b);
+
+		if(!fsdbReaderGotoNextVC(GLOBALS->extload_ffr_ctx, hdl))
+			{
+			break;
+			}
+			
+		mxt = (TimeType)fsdbReaderGetXTag(GLOBALS->extload_ffr_ctx, hdl);
+		}
+	}
+fsdbReaderFree(GLOBALS->extload_ffr_ctx, hdl);
+/* fsdbReaderUnloadSignals(GLOBALS->extload_ffr_ctx); */
+}
+
+#else
+
 if(last_modification_check()) /* place array height check here in an "&&" branch, sorry, arrays not supported */
 	{
 	char sbuff[65537];
@@ -985,6 +1375,7 @@ if(last_modification_check()) /* place array height check here in an "&&" branch
 
 	pclose(GLOBALS->extload);
 	}
+#endif
 
 histent_tail = htemp = histent_calloc();
 if(len>1)
@@ -1066,5 +1457,37 @@ if(nold!=np)
 	}
 }
 
-#endif
 
+void fsdb_import_masked(void)
+{
+#ifdef WAVE_FSDB_READER_IS_PRESENT
+fsdbReaderLoadSignals(GLOBALS->extload_ffr_ctx);
+GLOBALS->extload_ffr_import_count = 0;
+#endif
+}
+
+void fsdb_set_fac_process_mask(nptr np)
+{
+#ifdef WAVE_FSDB_READER_IS_PRESENT
+struct fac *f;
+int txidx, txidx_in_trace;
+
+if(!(f=np->mv.mvlfac)) return;	/* already imported */
+
+txidx = f - GLOBALS->mvlfacs_vzt_c_3;
+txidx_in_trace = GLOBALS->extload_idcodes[txidx];
+
+if(GLOBALS->extload_inv_idcodes[txidx_in_trace] > 0)
+	{
+	if(!GLOBALS->extload_ffr_import_count)
+		{
+		fsdbReaderUnloadSignals(GLOBALS->extload_ffr_ctx);
+		}
+	GLOBALS->extload_ffr_import_count++;
+
+	fsdbReaderAddToSignalList(GLOBALS->extload_ffr_ctx, txidx_in_trace);
+	}
+#endif
+}
+
+#endif
