@@ -168,6 +168,13 @@ while((h = fstReaderIterateHier(xc)))
 				case FST_ST_VCD_FUNCTION:	ttype = TREE_VCD_ST_FUNCTION; break;
 				case FST_ST_VCD_BEGIN:		ttype = TREE_VCD_ST_BEGIN; break;
 				case FST_ST_VCD_FORK:		ttype = TREE_VCD_ST_FORK; break;
+				case FST_ST_VCD_GENERATE:	ttype = TREE_VCD_ST_GENERATE; break;
+				case FST_ST_VCD_STRUCT:		ttype = TREE_VCD_ST_STRUCT; break;
+				case FST_ST_VCD_UNION:		ttype = TREE_VCD_ST_UNION; break;
+				case FST_ST_VCD_CLASS:		ttype = TREE_VCD_ST_CLASS; break;
+				case FST_ST_VCD_INTERFACE:	ttype = TREE_VCD_ST_INTERFACE; break;
+				case FST_ST_VCD_PACKAGE:	ttype = TREE_VCD_ST_PACKAGE; break;
+				case FST_ST_VCD_PROGRAM:	ttype = TREE_VCD_ST_PROGRAM; break;
 				default:			ttype = TREE_UNKNOWN; break;
 				}
 
@@ -178,6 +185,7 @@ while((h = fstReaderIterateHier(xc)))
                         GLOBALS->fst_scope_name = fstReaderPopScope(xc);
 			GLOBALS->fst_scope_name_len = fstReaderGetCurrentScopeLen(xc);
                         break;
+
                 case FST_HT_VAR:
                         /* GLOBALS->fst_scope_name = fstReaderGetCurrentFlatScope(xc); */
 			/* GLOBALS->fst_scope_name_len = fstReaderGetCurrentScopeLen(xc); */
@@ -207,6 +215,12 @@ while((h = fstReaderIterateHier(xc)))
 				{
 				*msb = *lsb = -1;
 				}
+			else if	((h->u.var.length > 1) && !col_last && lb_last) /* add for NC arrays that don't add final explicit bitrange like VCS does */
+				{
+				lb_last = NULL;
+				*msb = h->u.var.length -1;
+				*lsb = 0;
+				}
 			else
 				{
 				int sgna = 1, sgnb = 1;
@@ -263,6 +277,13 @@ while((h = fstReaderIterateHier(xc)))
 			*nam = s;
 			return(h);
                         break;
+
+		case FST_HT_ATTRBEGIN:	/* currently ignored */
+		case FST_HT_ATTREND:
+			break;
+
+		default:
+			break;
                 }
         }
 
@@ -420,7 +441,7 @@ for(i=0;i<GLOBALS->numfacs;i++)
 	char *str;	
 	struct fac *f;
 	int hier_len, name_len, tlen;
-	unsigned char nvt;
+	unsigned char nvt, nvd;
 	int longest_nam_candidate = 0;
 	char *fnam;
 
@@ -485,6 +506,15 @@ for(i=0;i<GLOBALS->numfacs;i++)
 
 	if(h->u.var.length)
 		{
+		switch(h->u.var.direction)
+			{
+			case FST_VD_INPUT:		nvd = ND_DIR_IN; GLOBALS->nonimplicit_direction_encountered = 1; break;
+			case FST_VD_OUTPUT:		nvd = ND_DIR_OUT; GLOBALS->nonimplicit_direction_encountered = 1; break;
+			case FST_VD_INOUT:		nvd = ND_DIR_INOUT; GLOBALS->nonimplicit_direction_encountered = 1; break;
+			case FST_VD_IMPLICIT:
+			default:			nvd = ND_DIR_IMPLICIT; break;
+			}
+
 		switch(h->u.var.typ)
 			{
 	                case FST_VT_VCD_EVENT: 		nvt = ND_VCD_EVENT; break;
@@ -507,7 +537,18 @@ for(i=0;i<GLOBALS->numfacs;i++)
 	                case FST_VT_VCD_WIRE: 		nvt = ND_VCD_WIRE; break;
 	                case FST_VT_VCD_WOR: 		nvt = ND_VCD_WOR; break;
 	                case FST_VT_VCD_PORT: 		nvt = ND_VCD_PORT; break;
+
 	                case FST_VT_GEN_STRING:		nvt = ND_GEN_STRING; break;
+
+			case FST_VT_SV_BIT:             nvt = ND_SV_BIT; break;
+			case FST_VT_SV_LOGIC:           nvt = ND_SV_LOGIC; break;
+			case FST_VT_SV_INT:             nvt = ND_SV_INT; break;
+			case FST_VT_SV_SHORTINT:        nvt = ND_SV_SHORTINT; break;
+			case FST_VT_SV_LONGINT:         nvt = ND_SV_LONGINT; break;
+			case FST_VT_SV_BYTE:            nvt = ND_SV_BYTE; break;
+			case FST_VT_SV_ENUM:            nvt = ND_SV_ENUM; break;
+			case FST_VT_SV_SHORTREAL:       nvt = ND_SV_SHORTREAL; break;
+
 			default: 			nvt = ND_UNSPECIFIED_DEFAULT; break;
 			}
 
@@ -521,6 +562,7 @@ for(i=0;i<GLOBALS->numfacs;i++)
 			case FST_VT_VCD_REAL:
 			case FST_VT_VCD_REAL_PARAMETER:
 			case FST_VT_VCD_REALTIME:
+			case FST_VT_SV_SHORTREAL:
 				GLOBALS->mvlfacs_fst_c_3[i].flags = VZT_RD_SYM_F_DOUBLE;
 				break;
 
@@ -706,6 +748,7 @@ for(i=0;i<GLOBALS->numfacs;i++)
         n->mv.mvlfac = GLOBALS->mvlfacs_fst_c_3+i;
 	GLOBALS->mvlfacs_fst_c_3[i].working_node = n;
 	n->vartype = nvt;
+	n->vardir = nvd;
 
 	if((f->len>1)||(f->flags&(VZT_RD_SYM_F_DOUBLE|VZT_RD_SYM_F_STRING)))
 		{
