@@ -47,6 +47,8 @@
 #include "fst.h"
 #include "hierpack.h"
 
+#include "fsdb_wrapper_api.h"
+
 #ifdef __MINGW32__
 #define sleep(x) Sleep(x)
 #endif
@@ -226,11 +228,13 @@ NULL, /* atoi_cont_ptr 78 */
 0, /* entry_entry_c_1 81 */
 NULL, /* entrybox_text 82 */
 0, /* cleanup_entry_c_1 83 */
-
+0, /* entry_raise_timer */
 
 /* 
  * extload.c 
  */
+0, /* extload_ffr_import_count */
+NULL, /* extload_ffr_ctx */
 NULL, /* extload */
 NULL, /* extload_idcodes */
 NULL, /* extload_inv_idcodes */
@@ -238,6 +242,17 @@ NULL, /* extload_inv_idcodes */
 0, /* extload_lastmod */
 0, /* extload_already_errored */
 #endif
+NULL, /* extload_namecache */
+NULL, /* extload_sym_block */
+NULL, /* extload_node_block */
+NULL, /* extload_xc */
+NULL, /* extload_prevsymroot */
+NULL, /* extload_prevsym */
+0, /* extload_i */
+0, /* extload_hlen */
+0, /* extload_vt_prev */
+0, /* extload_vd_prev */
+
 
 /*
  * fetchbuttons.c
@@ -298,6 +313,7 @@ NULL, /* mvlfacs_fst_rvs_alias */
 0, /* busycnt_fst_c_2 */
 NULL, /* double_curr_fst */
 NULL, /* double_fini_fst */
+0, /* nonimplicit_direction_encountered */
 
 
 /*
@@ -647,6 +663,14 @@ NULL, /* hiericon_begin_pixmap */
 NULL, /* hiericon_begin_mask */
 NULL, /* hiericon_fork_pixmap */
 NULL, /* hiericon_fork_mask */
+NULL, /* hiericon_interface_pixmap */
+NULL, /* hiericon_interface_mask */
+NULL, /* hiericon_svpackage_pixmap */
+NULL, /* hiericon_svpackage_mask */
+NULL, /* hiericon_program_pixmap */
+NULL, /* hiericon_program_mask */
+NULL, /* hiericon_class_pixmap */
+NULL, /* hiericon_class_mask */
 NULL, /* hiericon_design_pixmap */
 NULL, /* hiericon_design_mask */
 NULL, /* hiericon_block_pixmap */
@@ -1976,14 +2000,24 @@ void reload_into_new_context_2(void)
 #endif
 	break;
   
+#ifdef EXTLOAD_SUFFIX
+   case EXTLOAD_FILE:
+	if(GLOBALS->extload_ffr_ctx)
+		{
+#ifdef WAVE_FSDB_READER_IS_PRESENT
+		fsdbReaderClose(GLOBALS->extload_ffr_ctx);
+#endif
+		GLOBALS->extload_ffr_ctx = NULL;
+		}
+	break;
+#endif
+
    case MISSING_FILE:
    case DUMPLESS_FILE:
    case GHW_FILE:
    case VCD_FILE:
    case VCD_RECODER_FILE: 
-#ifdef EXTLOAD_SUFFIX
-   case EXTLOAD_FILE:
-#endif
+   default:
 	/* do nothing */ break;
  }
 
@@ -2520,15 +2554,25 @@ void free_and_destroy_page_context(void)
 #endif
 #endif
         break;
+
+#ifdef EXTLOAD_SUFFIX
+   case EXTLOAD_FILE:
+	if(GLOBALS->extload_ffr_ctx)
+		{
+#ifdef WAVE_FSDB_READER_IS_PRESENT
+		fsdbReaderClose(GLOBALS->extload_ffr_ctx);
+#endif
+		GLOBALS->extload_ffr_ctx = NULL;
+		}
+	break;
+#endif
  
    case MISSING_FILE:
    case DUMPLESS_FILE:
    case GHW_FILE:
    case VCD_FILE:
    case VCD_RECODER_FILE: 
-#ifdef EXTLOAD_SUFFIX   
-   case EXTLOAD_FILE:
-#endif          
+   default:
 	/* do nothing */ break;
  }
 
@@ -2813,7 +2857,14 @@ void clone_icon_pointers_across_contexts(struct Global *a, struct Global *b)
  a->hiericon_begin_mask = b->hiericon_begin_mask;  
  a->hiericon_fork_pixmap = b->hiericon_fork_pixmap;
  a->hiericon_fork_mask = b->hiericon_fork_mask;
- 
+ a->hiericon_interface_pixmap = b->hiericon_interface_pixmap;
+ a->hiericon_interface_mask = b->hiericon_interface_mask;
+ a->hiericon_svpackage_pixmap = b->hiericon_svpackage_pixmap;
+ a->hiericon_svpackage_mask = b->hiericon_svpackage_mask;
+ a->hiericon_program_pixmap = b->hiericon_program_pixmap;
+ a->hiericon_program_mask = b->hiericon_program_mask;
+ a->hiericon_class_pixmap = b->hiericon_class_pixmap;
+ a->hiericon_class_mask = b->hiericon_class_mask;
  a->hiericon_design_pixmap = b->hiericon_design_pixmap;
  a->hiericon_design_mask = b->hiericon_design_mask;  
  a->hiericon_block_pixmap = b->hiericon_block_pixmap;
