@@ -228,9 +228,12 @@ GtkWidget *window_entry_c_1; /* from entry.c 81 */
 GtkWidget *entry_entry_c_1; /* from entry.c 82 */
 char *entrybox_text; /* from entry.c 83 */
 void (*cleanup_entry_c_1)(void); /* from entry.c 84 */
+int entry_raise_timer;
 
 
 /* extload.c */
+unsigned int extload_ffr_import_count; /* from extload.c */
+void *extload_ffr_ctx; /* from extload.c */
 FILE *extload; /* from extload.c */
 unsigned int *extload_idcodes; /* from extload.c */
 int *extload_inv_idcodes; /* from extload.c */
@@ -238,6 +241,16 @@ int *extload_inv_idcodes; /* from extload.c */
 time_t extload_lastmod; /* from extload.c */
 char extload_already_errored; /* from extload.c */
 #endif
+char **extload_namecache;
+struct symbol *extload_sym_block;
+struct Node *extload_node_block;
+void *extload_xc;
+struct symbol *extload_prevsymroot;
+struct symbol *extload_prevsym;
+int extload_i;
+int extload_hlen;
+unsigned char extload_vt_prev;
+unsigned char extload_vd_prev;
 
 
 /*
@@ -299,6 +312,7 @@ fstHandle fst_maxhandle;
 int busycnt_fst_c_2; 
 double *double_curr_fst;
 double *double_fini_fst;
+char nonimplicit_direction_encountered;
 
 
 /*
@@ -646,6 +660,14 @@ GdkDrawable *hiericon_begin_pixmap; /* from pixmaps.c */
 GdkDrawable *hiericon_begin_mask; /* from pixmaps.c */
 GdkDrawable *hiericon_fork_pixmap; /* from pixmaps.c */
 GdkDrawable *hiericon_fork_mask; /* from pixmaps.c */
+GdkDrawable *hiericon_interface_pixmap;
+GdkDrawable *hiericon_interface_mask;
+GdkDrawable *hiericon_svpackage_pixmap;
+GdkDrawable *hiericon_svpackage_mask;
+GdkDrawable *hiericon_program_pixmap;
+GdkDrawable *hiericon_program_mask;
+GdkDrawable *hiericon_class_pixmap;
+GdkDrawable *hiericon_class_mask;
 GdkDrawable *hiericon_design_pixmap;
 GdkDrawable *hiericon_design_mask;
 GdkDrawable *hiericon_block_pixmap;
