@@ -46,21 +46,57 @@
 #define VCD2FST_EXTLOADERS_CONV
 #endif
 
+static uint32_t var_direction_idx = 0;
+static unsigned char *var_direction = NULL;
+
 /*********************************************************/
 /*** vvv extload component type name determination vvv ***/
 /*********************************************************/
 
 #if defined(VCD2FST_EXTLOAD_CONV)
 
-#include <Judy.h>
-
 #ifdef _WAVE_HAVE_JUDY
+#include <Judy.h>
 Pvoid_t  PJArray = NULL;
 #else
 JRB comp_name_jrb = NULL;
 #endif
 
 static const char *fst_scope_name = NULL;
+static uint32_t numfacs = 0;
+
+static char *get_info(FILE *extload)
+{
+static char sbuff[65537];
+char * rc;
+
+for(;;)
+        {
+        rc = fgets(sbuff, 65536, extload);
+	if(!rc)
+                {
+                return(NULL);
+                }
+
+	switch(rc[0])
+		{
+                case 'v':
+                        if(!strncmp("var creation cnt", rc, 16))
+                                {
+                                char *pnt = strchr(rc+16, ':');
+                                if(pnt)
+                                        {
+                                        pnt++;
+                                        sscanf(pnt, "%d", &numfacs);
+                                        }
+                                }
+			break;
+
+		default:
+			break;			
+		}
+	}
+}
 
 static char *get_scopename(void *xc, FILE *extload)
 {
@@ -76,11 +112,65 @@ Jval jv;
 for(;;)
         {
         rc = fgets(sbuff, 65536, extload);
-        if(!rc)
+        if(rc)
+		{
+                if(isspace(rc[0]))
+                        {
+                        char sbuff2[65537];
+
+                        sbuff2[0] = 0;
+
+                        if(strstr(rc+1, "Struct Name:"))
+                                {
+                                sscanf(rc+14,"%s", sbuff2);
+                                if(sbuff2[0])
+                                        {
+                                        sprintf(rc, "Scope: vcd_struct %s NULL\n", sbuff2);
+                                        }
+                                }
+                        else
+                        if(strstr(rc+1, "Struct End"))
+                                {
+                                sprintf(rc, "Upscope:\n");  
+                                }
+			}
+		}
+	else
                 {
                 return(NULL);
                 }
 
+	if(rc[0] == 'V')
+		{
+                if(!strncmp("Var: ", rc, 5))
+                        {
+                        char *pnt = rc + 5;
+                        char *pntd = strrchr(pnt, ':');
+
+                        if(pntd)
+                                {
+                                unsigned char vd = FST_VD_IMPLICIT;
+
+                                pntd = strchr(pntd, ' ');
+                                if(pntd)
+                                        {
+                                        pntd++;
+                                        if(*pntd == 'o')
+                                                {
+                                                vd = FST_VD_OUTPUT;
+                                                }
+                                        else
+                                        if(!strncmp(pntd, "in", 2))
+                                                {
+                                                vd = (pntd[2] == 'p') ? FST_VD_INPUT : FST_VD_INOUT;
+                                                }
+                                        }
+
+				var_direction[var_direction_idx++] = vd;
+                                }
+			}
+		}
+	else
         if(rc[0] == 'S')
                 {
                 if(!strncmp(rc, "Scope:", 6))
@@ -88,14 +178,36 @@ for(;;)
                         char vht[2048];
                         char cname[2048];
                         char ctype[2048];
+			int mtype = FST_ST_VCD_MODULE;
 
-			cname[0] = ctype[0] = 0;
+			cname[0] = ctype[1] = 0;
+
+                        sscanf(rc+6, "%s %s %s", vht, cname, ctype+1);
+			if(!strncmp("vcd_", vht, 4))
+				{
+				switch(vht[4])
+					{
+					case 'g':	mtype = FST_ST_VCD_GENERATE; break; /* other code looks for non-modules to replace type with */
+					case 's':	mtype = FST_ST_VCD_STRUCT;   break; /* other code looks for non-modules to replace type with */
+					default:	break;
+					}
+				}
+			else
+			if(!strncmp("sv_", vht, 3))
+				{
+				switch(vht[3])
+					{
+					case 'i':	mtype = FST_ST_VCD_INTERFACE; break; /* other code looks for non-modules to replace type with */
+					default:	break;
+					}
+				}
+
+			ctype[0] = mtype + 1; /* bias for zero terminated string */
 
-                        sscanf(rc+6, "%s %s %s", vht, cname, ctype);
                         fst_scope_name = fstReaderPushScope(xc, cname, NULL);
 
 			/* process fst_scope_name + cname vs ctype here */
-			if(strcmp(ctype, "NULL") && strcmp(cname, ctype))
+			if((strcmp(ctype+1, "NULL") && strcmp(cname, ctype+1)) || (mtype != FST_ST_VCD_MODULE))
 				{
 #ifdef _WAVE_HAVE_JUDY
 				PPValue = JudySLIns(&PJArray, (uint8_t *)fst_scope_name, PJE0);
@@ -133,12 +245,30 @@ char sbuff[65537];
 FILE *extload;
 void *xc = fstReaderOpenForUtilitiesOnly();
 
-sprintf(sbuff, "%s -scope %s 2>&1", EXTLOAD_PATH, fname);
+sprintf(sbuff, "%s -info %s 2>&1", EXTLOAD_PATH, fname);
 extload = popen(sbuff, "r");
+if(extload)
+	{
+	while(get_info(extload));
+	pclose(extload);
+	}
 
-while(get_scopename(xc, extload));
+if(numfacs)
+	{
+	var_direction = calloc(numfacs, sizeof(unsigned char));
+	var_direction_idx = 0;
+	}
+
+sprintf(sbuff, "%s -tree %s 2>&1", EXTLOAD_PATH, fname);
+extload = popen(sbuff, "r");
+if(extload)
+	{
+	while(get_scopename(xc, extload));
+	pclose(extload);
+	}
+
+var_direction_idx = 0;
 
-pclose(extload);
 fstReaderClose(xc); /* corresponds to fstReaderOpenForUtilitiesOnly() */
 }
 
@@ -398,11 +528,51 @@ while(!feof(f))
 				break;
 
 			case 'i':
-				vartype = FST_VT_VCD_INTEGER;
+				if(!strcmp(st, "integer"))		
+					{
+					vartype = FST_VT_VCD_INTEGER;
+					}
+				else
+				if(!strcmp(st, "int"))		
+					{
+					vartype = FST_VT_SV_INT;
+					}
 				break;
 
 			case 'e':
-				vartype = FST_VT_VCD_EVENT;
+				if(!strcmp(st, "event"))
+					{	
+					vartype = FST_VT_VCD_EVENT;
+					}
+				else
+				if(!strcmp(st, "enum"))
+					{	
+					vartype = FST_VT_SV_ENUM;
+					}
+				break;
+
+			case 'b':
+				if(!strcmp(st, "bit"))
+					{
+					vartype = FST_VT_SV_BIT;
+					}
+				else
+				if(!strcmp(st, "byte"))
+					{
+					vartype = FST_VT_SV_BYTE;
+					}
+				break;
+
+			case 'l':
+				if(!strcmp(st, "logic"))
+					{
+					vartype = FST_VT_SV_LOGIC;
+					}
+				else
+				if(!strcmp(st, "longint"))
+					{
+					vartype = FST_VT_SV_LONGINT;
+					}
 				break;
 
 			case 's':
@@ -420,6 +590,21 @@ while(!feof(f))
 					{
 					vartype = FST_VT_GEN_STRING;
 					}
+				else
+				if(!strcmp(st, "shortint"))		
+					{
+					vartype = FST_VT_SV_SHORTINT;
+					}
+				else
+				if(!strcmp(st, "shortreal"))		
+					{
+					vartype = FST_VT_SV_SHORTREAL;
+					}
+				else
+				if(!strcmp(st, "sparray"))		
+					{
+					vartype = FST_VT_VCD_SPARRAY;
+					}
 				break;
 
 			case 't':
@@ -506,14 +691,25 @@ while(!feof(f))
 			if(!node)
 				{
 				Jval val;
-				returnedhandle = fstWriterCreateVar(ctx, vartype, FST_VD_IMPLICIT, len, nam, 0);
+				returnedhandle = fstWriterCreateVar(ctx, vartype, !var_direction ? FST_VD_IMPLICIT : var_direction[var_direction_idx++], len, nam, 0);
 				val.i = returnedhandle;
 				jrb_insert_int(vcd_ids, hash, val)->val2.i = len;
 				}
 				else
 				{
-				fstWriterCreateVar(ctx, vartype, FST_VD_IMPLICIT, node->val2.i, nam, node->val.i);
+				fstWriterCreateVar(ctx, vartype, !var_direction ? FST_VD_IMPLICIT : var_direction[var_direction_idx++], node->val2.i, nam, node->val.i);
 				}
+
+#if defined(VCD2FST_EXTLOAD_CONV)
+			if(var_direction)
+				{
+				if(var_direction_idx == numfacs)
+					{
+					free(var_direction);
+					var_direction = NULL;
+					}
+				}
+#endif
 			}
 		}
 	else
@@ -546,6 +742,41 @@ while(!feof(f))
 			{
 			scopetype = FST_ST_VCD_FORK;
 			}
+		else
+		if(!strcmp(st, "generate"))
+			{
+			scopetype = FST_ST_VCD_GENERATE;
+			}
+		else
+		if(!strcmp(st, "struct"))
+			{
+			scopetype = FST_ST_VCD_STRUCT;
+			}
+		else
+		if(!strcmp(st, "union"))
+			{
+			scopetype = FST_ST_VCD_UNION;
+			}
+		else
+		if(!strcmp(st, "class"))
+			{
+			scopetype = FST_ST_VCD_CLASS;
+			}
+		else
+		if(!strcmp(st, "interface"))
+			{
+			scopetype = FST_ST_VCD_INTERFACE;
+			}
+		else
+		if(!strcmp(st, "package"))
+			{
+			scopetype = FST_ST_VCD_PACKAGE;
+			}
+		else
+		if(!strcmp(st, "program"))
+			{
+			scopetype = FST_ST_VCD_PROGRAM;
+			}
 		else
 			{
 			scopetype = FST_ST_VCD_MODULE;
@@ -557,22 +788,58 @@ while(!feof(f))
 #ifdef _WAVE_HAVE_JUDY
 		if(PJArray)
 			{
-			const char *fst_scope_name = fstReaderPushScope(xc, st, NULL);
-			PPvoid_t PPValue = JudySLGet(PJArray, (uint8_t *)fst_scope_name, PJE0);
+			const char *fst_scope_name2 = fstReaderPushScope(xc, st, NULL);
+			PPvoid_t PPValue = JudySLGet(PJArray, (uint8_t *)fst_scope_name2, PJE0);
 
-			fstWriterSetScope(ctx, scopetype, st, PPValue ? *PPValue : NULL);
+			if(PPValue)
+				{
+				unsigned char st_replace = (*((unsigned char *)*PPValue)) - 1;
+				if(st_replace != FST_ST_VCD_MODULE)
+					{
+					scopetype = st_replace;
+					}
+
+				if((scopetype == FST_ST_VCD_GENERATE)||(scopetype == FST_ST_VCD_STRUCT))
+					{
+					PPValue = NULL;
+					}
+
+				fstWriterSetScope(ctx, scopetype, st, PPValue ? ((char *)(*PPValue)+1) : NULL);
+				}
+				else
+				{
+				fstWriterSetScope(ctx, scopetype, st, NULL);
+				}
 			}
 #else
 		if(comp_name_jrb)
 			{
-			const char *fst_scope_name = fstReaderPushScope(xc, st, NULL);
+			const char *fst_scope_name2 = fstReaderPushScope(xc, st, NULL);
 			char cstring[65537];
 			JRB str;
 
-			strcpy(cstring, fst_scope_name);
+			strcpy(cstring, fst_scope_name2);
 			str = jrb_find_str(comp_name_jrb, cstring);
 
-			fstWriterSetScope(ctx, scopetype, st, str ? str->val.s : NULL);
+			if(str)
+				{
+				unsigned char st_replace = str->val.s[0] - 1;
+				if(st_replace != FST_ST_VCD_MODULE)
+					{
+					scopetype = st_replace;
+					}
+
+				if((scopetype == FST_ST_VCD_GENERATE)||(scopetype == FST_ST_VCD_STRUCT))
+					{
+					str = NULL;
+					}
+
+				fstWriterSetScope(ctx, scopetype, st, str ? (str->val.s+1) : NULL);
+				}
+				else
+				{
+				fstWriterSetScope(ctx, scopetype, st, NULL);
+				}
 			}
 #endif
 			else
@@ -744,9 +1011,10 @@ while(!feof(f))
 		fstWriterSetDate(ctx, pnt);
 		}
 	else
-	if(!strncmp(buf, "$version", 8))
+	if((!strncmp(buf, "$version", 8)) || (!strncmp(buf, "$comment", 8)))
 		{
 		char *pnt, *crpnt, *rsp;
+		int is_version = (buf[1] == 'v');
 
                 if((pnt = strstr(buf, "$end")))
                         {
@@ -781,7 +1049,15 @@ while(!feof(f))
 		if(crpnt) *crpnt = 0;
 		crpnt = strchr(pnt, '\r');
 		if(crpnt) *crpnt = 0;
-		fstWriterSetVersion(ctx, pnt);
+
+		if(is_version)
+			{
+			fstWriterSetVersion(ctx, pnt);
+			}
+			else
+			{
+			fstWriterSetComment(ctx, pnt);
+			}
 		}
 	}
 
@@ -1282,6 +1558,10 @@ if (optind < argc)
                         lxname = malloc(strlen(argv[optind])+1);
                         strcpy(lxname, argv[optind++]);
                         }
+		else
+			{
+			break;
+			}
                 }
         }
                         
