@@ -261,6 +261,9 @@ static void entrybox_local(char *title, int width, char *default_text, int maxch
 
     GLOBALS->cleanup_e_hiersearch_c_1=func;
 
+    /* fix problem where ungrab doesn't occur if button pressed + simultaneous accelerator key occurs */
+    if(GLOBALS->in_button_press_wavewindow_c_1) { gdk_pointer_ungrab(GDK_CURRENT_TIME); }
+
     /* create a new modal window */
     GLOBALS->window1_hiersearch_c_1 = gtk_window_new(GLOBALS->disable_window_manager ? GTK_WINDOW_POPUP : GTK_WINDOW_TOPLEVEL);
     install_focus_cb(GLOBALS->window1_hiersearch_c_1, ((char *)&GLOBALS->window1_hiersearch_c_1) - ((char *)GLOBALS));
@@ -870,6 +873,9 @@ void hier_searchbox(char *title, GtkSignalFunc func)
     GtkWidget *table;
     GtkTooltips *tooltips;
 
+    /* fix problem where ungrab doesn't occur if button pressed + simultaneous accelerator key occurs */
+    if(GLOBALS->in_button_press_wavewindow_c_1) { gdk_pointer_ungrab(GDK_CURRENT_TIME); }
+
     if(GLOBALS->is_active_hiersearch_c_1) 
 	{
 	gdk_window_raise(GLOBALS->window_hiersearch_c_3->window);
