@@ -21,6 +21,10 @@
 #include <windows.h>
 #endif
 
+/*
+#define WAVE_CRASH_ON_GTK_WARNING
+*/
+
 #include "wave_locale.h"
 
 #if !defined _MSC_VER && !defined __MINGW32__
@@ -683,6 +687,10 @@ if(!mainwindow_already_built)
 		fprintf(stderr, "Could not initialize GTK!  Is DISPLAY env var/xhost set?\n\n");
 		print_help(argv[0]);
 		}
+
+#ifdef WAVE_CRASH_ON_GTK_WARNING
+	g_log_set_always_fatal(G_LOG_LEVEL_CRITICAL|G_LOG_LEVEL_WARNING);
+#endif
 	}
 
 #if defined(__APPLE__)
@@ -1590,6 +1598,7 @@ if((wname)||(vcd_save_handle_cached)||(is_smartsave))
 				case LXT2_IS_VZT:  vzt_import_masked(); break;
 				case LXT2_IS_VLIST: vcd_import_masked(); break;
 				case LXT2_IS_FST: fst_import_masked(); break;
+				case LXT2_IS_FSDB: fsdb_import_masked(); break;
 				}
 
 			if(wave_is_compressed)
@@ -2238,7 +2247,7 @@ if(GLOBALS->treeopen_chain_head)
 		{
 		if(GLOBALS->ctree_main)
 			{
-			force_open_tree_node(t->str);
+			force_open_tree_node(t->str, 0);
 			}
 
 		t2 = t->next;
@@ -2536,7 +2545,7 @@ else
 #endif
 if(is_interactive)
 	{
-	for(;;) kick_partial_vcd();
+	for(;;) { kick_partial_vcd(); }
 	}
 	else
 	{
