@@ -200,6 +200,8 @@ WV_MENU_SSR,
 WV_MENU_SSH,
 WV_MENU_SST,
 WV_MENU_SEP7,
+WV_MENU_OPENH,
+WV_MENU_SEP7D,
 WV_MENU_ACOL,
 WV_MENU_ACOLR,
 WV_MENU_ABON,
