@@ -2304,6 +2304,600 @@ static char *arrow_divide[] = {
 };
 
 
+/* XPM */
+static char *arrow_inout[] = {
+/* columns rows colors chars-per-pixel */
+"16 16 109 2",
+"   c #418A3E",
+".  c #428B3F",
+"X  c #428C3F",
+"o  c #438D40",
+"O  c #448D40",
+"+  c #458E41",
+"@  c #458F41",
+"#  c #469042",
+"$  c #479143",
+"%  c #499244",
+"&  c #499445",
+"*  c #4A9545",
+"=  c #4B9646",
+"-  c #4C9747",
+";  c #4E974A",
+":  c #4D9948",
+">  c #4E9948",
+",  c #4F9B4A",
+"<  c #509C4A",
+"1  c #519E4B",
+"2  c #529F4C",
+"3  c #5A9C56",
+"4  c #5B9E57",
+"5  c #54A14E",
+"6  c #55A24E",
+"7  c #56A34F",
+"8  c #57A450",
+"9  c #57A550",
+"0  c #58A551",
+"q  c #58A651",
+"w  c #59A752",
+"e  c #5AA852",
+"r  c #5AA853",
+"t  c #5BA953",
+"y  c #5BAA54",
+"u  c #5CAA54",
+"i  c #5CAB54",
+"p  c #5DAB55",
+"a  c #5EAA57",
+"s  c #5DAC55",
+"d  c #5EAD56",
+"f  c #5FAD56",
+"g  c #5FAE57",
+"h  c #5EA15A",
+"j  c #60A15C",
+"k  c #62A45F",
+"l  c #60AF58",
+"z  c #61B058",
+"x  c #61B159",
+"c  c #62B159",
+"v  c #62B259",
+"b  c #63B35A",
+"n  c #64B35A",
+"m  c #64B25B",
+"M  c #64B45B",
+"N  c #64B05C",
+"B  c #66B65C",
+"V  c #67B75D",
+"C  c #68B95E",
+"Z  c #69B95F",
+"A  c #68A963",
+"S  c #6DB167",
+"D  c #6ABB60",
+"F  c #6BBC60",
+"G  c #6CBD61",
+"H  c #6CBE62",
+"J  c #6DBF62",
+"K  c #6DBF63",
+"L  c #6EB168",
+"P  c #6FB369",
+"I  c #77B96F",
+"U  c #75B071",
+"Y  c #77B373",
+"T  c #79B374",
+"R  c #77B871",
+"E  c #78BB70",
+"W  c #79BB71",
+"Q  c #79B972",
+"!  c #7ABA73",
+"~  c #7BB874",
+"^  c #7EB679",
+"/  c #6EC063",
+"(  c #70C365",
+")  c #71C365",
+"_  c #72C466",
+"`  c #72C566",
+"'  c #72C467",
+"]  c #73C667",
+"[  c #73C269",
+"{  c #74C768",
+"}  c #85BF7D",
+"|  c #87CA7E",
+" . c #86BD80",
+".. c #89BF83",
+"X. c #8AC184",
+"o. c #8BC186",
+"O. c #8DC387",
+"+. c #8ACD80",
+"@. c #8BCE81",
+"#. c #8CCC83",
+"$. c #90CF86",
+"%. c #92C68C",
+"&. c #93C78C",
+"*. c #94C78D",
+"=. c #98CD8E",
+"-. c #9FD197",
+";. c #A0D098",
+":. c #A7D69E",
+">. c None",
+/* pixels */
+"{ { ] ] ( >.>.>.>.>.>.B M c l d ",
+"{ @.$.( >.K >.>.>.>.B >.n W R u ",
+"] @.:.#.[ K >.>.>.>.n c W *.W r ",
+"( ( | =.-.F >.>.>.>.c &.} W 0 0 ",
+"( >.K -.-.Z >.>.>.>.g &.O.u >.8 ",
+">.K K F Z Z >.>.>.>.u t 0 0 6 >.",
+">.>.>.>.>.>.>.>.>.>.>.>.>.>.>.>.",
+">.>.>.>.>.>.>.>.>.>.>.>.>.>.>.>.",
+">.>.>.>.>.>.>.>.>.>.>.>.>.>.>.>.",
+">.>.>.>.>.>.>.>.>.>.>.>.>.>.>.>.",
+">.c c g d r >.>.>.>.1 < : ; * >.",
+"B >.c X.o.r >.>.>.>., ^ T * >.$ ",
+"l g R ~  .8 >.>.>.>., T A h % @ ",
+"d R ..L 8 6 >.>.>.>.* ; k U 4 X ",
+"d L L 9 >.5 >.>.>.>.* >.@ h 3 X ",
+"0 9 8 6 1 >.>.>.>.>.>.@ @ @ X   "
+};
+
+
+/* XPM */
+static char *application[] = {
+/* columns rows colors chars-per-pixel */
+"16 16 90 1",
+"  c #5171A9",
+". c #5172A9",
+"X c #5373A9",
+"o c #5373AA",
+"O c #5475AB",
+"+ c #5475AC",
+"@ c #5676AD",
+"# c #5779AE",
+"$ c #587AB0",
+"% c #587BB1",
+"& c #5A7CB2",
+"* c #5C7EB4",
+"= c #5C7EB5",
+"- c #5E81B7",
+"; c #5F83B9",
+": c #6085BA",
+"> c #6185BB",
+", c #6387BD",
+"< c #6488BE",
+"1 c #668BC0",
+"2 c #688EC3",
+"3 c #6F90C1",
+"4 c #6C92C8",
+"5 c #6C94C9",
+"6 c #6E94CA",
+"7 c #6F96CB",
+"8 c #6F97CC",
+"9 c #7A9AC4",
+"0 c #7099CE",
+"q c #729ACF",
+"w c #739CD1",
+"e c #739DD2",
+"r c #759ED4",
+"t c #76A0D5",
+"y c #79A2D8",
+"u c #79A3D9",
+"i c #7AA4DA",
+"p c #7AA4DB",
+"a c #7AA6DC",
+"s c #7BA7DC",
+"d c #85ACDD",
+"f c #8DB1DD",
+"g c #92B9E8",
+"h c #DEE7F2",
+"j c #E4EAF3",
+"k c #E7EBF5",
+"l c #E8ECF6",
+"z c #EAEFF6",
+"x c #EAF0F7",
+"c c #EBF2F7",
+"v c #EDF3F7",
+"b c #ECF2F8",
+"n c #EFF3FA",
+"m c #EEF4F8",
+"M c #F0F5F9",
+"N c #F2F5FB",
+"B c #F1F6FB",
+"V c #F2F7FA",
+"C c #F2F7FB",
+"Z c #F3F7FB",
+"A c #F4F7FB",
+"S c #F4F8FB",
+"D c #F5F8FB",
+"F c #F5F9FB",
+"G c #F7F9FB",
+"H c #F6FAFB",
+"J c #F7FAFB",
+"K c #F6FBFB",
+"L c #F7FBFC",
+"P c #F7FCFC",
+"I c #F9FBFB",
+"U c #F8FBFC",
+"Y c #F9FBFC",
+"T c #F8FCFC",
+"R c #F9FCFC",
+"E c #F8FDFD",
+"W c #F9FDFD",
+"Q c #FAFCFC",
+"! c #FAFDFD",
+"~ c #FBFDFD",
+"^ c #FAFEFE",
+"/ c #FBFEFE",
+"( c #FCFDFD",
+") c #FCFEFE",
+"_ c #FDFEFE",
+"` c #FCFFFF",
+"' c #FDFFFF",
+"] c #FEFFFF",
+"[ c #FFFFFF",
+"{ c None",
+/* pixels */
+"{{{{{{{{{{{{{{{{",
+"{faaputreqq759{{",
+"dggggggggggggg3{",
+"sssssssaassssa1{",
+"p[[[[[[[[[[[[[,{",
+"u[UUGASmvzjjh[:{",
+"u[!IUGGSBmbxl[-{",
+"t[[I!!GSGBAbb[={",
+"r[!![!!PGGSCC[&{",
+"e[[[!!!!PIHPC[${",
+"8[!![!!!!!!PG[+{",
+"5[[[[[[!![!!![o{",
+"4[[[[[[[[[[[[[ {",
+"o21,,;*&$#+Oooo{",
+"{{{{{{{{{{{{{{{{",
+"{{{{{{{{{{{{{{{{"
+};
+
+
+/* XPM */
+static char *package[] = {
+/* columns rows colors chars-per-pixel */
+"16 16 176 2",
+"   c #C38322",
+".  c #C38423",
+"X  c #C48524",
+"o  c #C48625",
+"O  c #C58726",
+"+  c #C58928",
+"@  c #C68A28",
+"#  c #C78D2B",
+"$  c #C88E2C",
+"%  c #C88F2D",
+"&  c #C9902E",
+"*  c #C9922F",
+"=  c #CB9532",
+"-  c #CC9834",
+";  c #C8943B",
+":  c #CE9D39",
+">  c #DE9F32",
+",  c #D1A43F",
+"<  c #E2AA37",
+"1  c #E2A938",
+"2  c #E3AB3A",
+"3  c #E6B53A",
+"4  c #E6B43D",
+"5  c #E6B53F",
+"6  c #ECB638",
+"7  c #E9B53D",
+"8  c #F0BD3B",
+"9  c #C88E40",
+"0  c #CA904A",
+"q  c #D09753",
+"w  c #D2A540",
+"e  c #D3A842",
+"r  c #D4AA44",
+"t  c #D5AC45",
+"y  c #D5AD47",
+"u  c #D6B049",
+"i  c #D7B04A",
+"p  c #D7B24B",
+"a  c #D8B24B",
+"s  c #D8B44C",
+"d  c #D8B44D",
+"f  c #D9B54D",
+"g  c #D9B54E",
+"h  c #D9B74F",
+"j  c #D5A651",
+"k  c #DAAD50",
+"l  c #D7A45F",
+"z  c #DBB156",
+"x  c #E1A740",
+"c  c #E6B542",
+"v  c #E4B145",
+"b  c #E6B147",
+"n  c #E7BA41",
+"m  c #E9BD40",
+"M  c #E8BE43",
+"N  c #E9BD45",
+"B  c #E4B14C",
+"V  c #E9B44C",
+"C  c #E8BF48",
+"Z  c #E8BB4F",
+"A  c #F5BD40",
+"S  c #E5AF56",
+"D  c #DBB262",
+"F  c #DDB969",
+"G  c #D6B870",
+"H  c #DCBD77",
+"J  c #E6B66B",
+"K  c #E2B76F",
+"L  c #E5BF6F",
+"P  c #E4B374",
+"I  c #EAC443",
+"U  c #EAC146",
+"Y  c #ECC547",
+"T  c #EBC648",
+"R  c #EFC748",
+"E  c #F9C248",
+"W  c #F9C448",
+"Q  c #F1C156",
+"!  c #F5C969",
+"~  c #E1C072",
+"^  c #E0C07D",
+"/  c #E0C67E",
+"(  c #F5CF7F",
+")  c #FFD577",
+"_  c #F5D078",
+"`  c #F5D07F",
+"'  c #FED678",
+"]  c #FED97C",
+"[  c #DBC389",
+"{  c #DEC995",
+"}  c #E5C482",
+"|  c #EBC182",
+" . c #ECC583",
+".. c #EEC483",
+"X. c #EDC687",
+"o. c #ECCA84",
+"O. c #EBCC84",
+"+. c #E7CC88",
+"@. c #EECA88",
+"#. c #EFCA8A",
+"$. c #EBCD8F",
+"%. c #F1CF86",
+"&. c #F0CE8A",
+"*. c #FBDC84",
+"=. c #F1D28B",
+"-. c #F7D589",
+";. c #F7D78B",
+":. c #F0D08C",
+">. c #FBDA8D",
+",. c #FFDC8C",
+"<. c #E5CF92",
+"1. c #EDD292",
+"2. c #EAD59C",
+"3. c #F1D590",
+"4. c #F5D790",
+"5. c #F2D695",
+"6. c #FBDB92",
+"7. c #FEDF95",
+"8. c #F8DB9D",
+"9. c #FEE094",
+"0. c #FEE194",
+"q. c #FEE196",
+"w. c #FFE19B",
+"e. c #FFE39F",
+"r. c #EDD9A5",
+"t. c #F2DDAF",
+"y. c #EDDEBE",
+"u. c #FFE4A4",
+"i. c #FFE4A5",
+"p. c #FFE5A5",
+"a. c #F9E1AA",
+"s. c #FAE3AA",
+"d. c #FFE7AD",
+"f. c #FFE8A8",
+"g. c #FFE8AB",
+"h. c #F7E4B2",
+"j. c #FAE4B0",
+"k. c #FFEAB3",
+"l. c #FFE9B6",
+"z. c #FFEBB7",
+"x. c #F6E5BF",
+"c. c #FFEAB9",
+"v. c #FFEBBA",
+"b. c #FFEBBD",
+"n. c #FFEDBE",
+"m. c #FFEEC1",
+"M. c #FFEFC1",
+"N. c #FFEFC4",
+"B. c #FCEEC7",
+"V. c #FBEDCB",
+"C. c #FFEFC8",
+"Z. c #F9EBCD",
+"A. c #FAECCE",
+"S. c #FFF0C9",
+"D. c #FFF2CC",
+"F. c #FFF2CE",
+"G. c #FFF4CC",
+"H. c #F5EBD2",
+"J. c #F7EED1",
+"K. c #FFF3D4",
+"L. c #FFF3D5",
+"P. c #FFF4D7",
+"I. c #FFF5D9",
+"U. c #FFF4DA",
+"Y. c #FFF6DB",
+"T. c #FFF6DD",
+"R. c #FCF6E2",
+"E. c #FDF6E2",
+"W. c #FEF6E3",
+"Q. c #FEF8E1",
+"!. c #FDF8E2",
+"~. c #FCF8E6",
+"^. c #FEFBE7",
+"/. c #FFFEF0",
+"(. c #FFFFF6",
+"). c None",
+/* pixels */
+").).).).).).).a p ).).).).).).).",
+").).).).).g p 2.+.r r ).).).).).",
+").).).).g <.~.R.t.j.~ w ).).).).",
+").).g / J.(./.V.k ;.a.5.D : ).).",
+"g g r.H.y.{ [ G F 4.( 8.a.o.j = ",
+"g Q.A.1.O.L z ; A.I.c.7.' p.*.* ",
+"p R.K.s.&.` _ B j.l.w.' A 7 3.% ",
+"p R.I.K.b.-.! V >.,.W 6 C Z :.# ",
+"y R.P.D.N.c.6.Q E 8 R N c b #.+ ",
+"r R.K.D.m.k.d.] R I M 4 1 x  .O ",
+"e ~.T.M.l.g.u.q.T m 4 1 > J  .o ",
+", H x.T.m.p.e.q.U 3 < S  .P 0   ",
+").).: ^ B.D.p.q.n v #.| q . ).).",
+").).).- = } G.N.:.&.l o o ).).).",
+").).).).).* % $.K 9 o ).).).).).",
+").).).).).).).+ O ).).).).).).)."
+};
+
+
+/* XPM */
+static char *plugin[] = {
+/* columns rows colors chars-per-pixel */
+"16 16 123 2",
+"   c #3A8A36",
+".  c #3D8C39",
+"X  c #3D8D39",
+"o  c #3F8E3A",
+"O  c #408F3B",
+"+  c #41903C",
+"@  c #42913D",
+"#  c #42923E",
+"$  c #43923E",
+"%  c #459440",
+"&  c #469641",
+"*  c #479642",
+"=  c #4A9945",
+"-  c #4D9C48",
+";  c #4E9D48",
+":  c #4F9E49",
+">  c #51A04B",
+",  c #52A14C",
+"<  c #55A44F",
+"1  c #59B54F",
+"2  c #59A752",
+"3  c #5AA953",
+"4  c #5DAB56",
+"5  c #5EAD57",
+"6  c #5BB651",
+"7  c #5DB752",
+"8  c #5FB853",
+"9  c #61AF5A",
+"0  c #62AC5C",
+"q  c #63AD5D",
+"w  c #60B955",
+"e  c #61B956",
+"r  c #61BA56",
+"t  c #63B15B",
+"y  c #65B45E",
+"u  c #67B55F",
+"i  c #63BA58",
+"p  c #64BB58",
+"a  c #64BB59",
+"s  c #66BB5A",
+"d  c #66BB5D",
+"f  c #68BC5B",
+"g  c #69BC5D",
+"h  c #69BD5D",
+"j  c #6ABD5E",
+"k  c #6CBE5F",
+"l  c #6CB565",
+"z  c #6DB566",
+"x  c #6AB862",
+"c  c #6BB963",
+"v  c #6BBD62",
+"b  c #6DBF60",
+"n  c #6EBC65",
+"m  c #6FBF65",
+"M  c #6FBD66",
+"N  c #6FC062",
+"B  c #71C164",
+"V  c #72C165",
+"C  c #73C265",
+"Z  c #71C066",
+"A  c #73C267",
+"S  c #72C069",
+"D  c #72C169",
+"F  c #77C369",
+"G  c #76C46D",
+"H  c #77C46D",
+"J  c #78C66F",
+"K  c #7AC072",
+"L  c #7AC770",
+"P  c #7CC376",
+"I  c #7BC971",
+"U  c #7CC972",
+"Y  c #7DCA73",
+"T  c #7DCB73",
+"R  c #7ECB74",
+"E  c #80CD75",
+"W  c #80CE76",
+"Q  c #81CE76",
+"!  c #81C67A",
+"~  c #81C67B",
+"^  c #83C77C",
+"/  c #87CB79",
+"(  c #82CD78",
+")  c #82CF78",
+"_  c #85C97C",
+"`  c #84C87E",
+"'  c #85C87E",
+"]  c #83D079",
+"[  c #87CA80",
+"{  c #88CA80",
+"}  c #88CA81",
+"|  c #89CA82",
+" . c #89CA83",
+".. c #8ACB83",
+"X. c #8BCC83",
+"o. c #8CCC84",
+"O. c #8DCD85",
+"+. c #8ECD86",
+"@. c #90CF88",
+"#. c #91CF88",
+"$. c #92CF8A",
+"%. c #93CF8B",
+"&. c #94CF8A",
+"*. c #94CF8B",
+"=. c #95CF8B",
+"-. c #95D08C",
+";. c #96D18D",
+":. c #97D28D",
+">. c #98D18F",
+",. c #99D290",
+"<. c #9CD493",
+"1. c #9DD593",
+"2. c #9ED595",
+"3. c #A1D796",
+"4. c #A1D797",
+"5. c #A2D897",
+"6. c #A2D798",
+"7. c #A2D898",
+"8. c #A3D899",
+"9. c #A4D899",
+"0. c #A6D99B",
+"q. c #A7D99C",
+"w. c None",
+/* pixels */
+"w.w.w.w.w.w.w.w.w.w.w.w.w.w.w.w.",
+"w.w.w.w.w.( W T L w.w.w.w.w.w.w.",
+"w.w.w.w.w.( q.5.H w.w.w.w.w.w.w.",
+"w.w.w.w.w.T 5.>.S w.w.w.w.w.w.w.",
+"w.] W W T H 4.1.n x u t w.w.w.w.",
+"w.W 9.9.4.4.1.<.>.-.*.5 w.w.w.w.",
+"w.T 8.4./ F C B V V *.3 w.w.w.w.",
+"w.L H _ <.V m k j s @.z , ; = w.",
+"w.w.w.M ,.k j s a r n X. .[ & w.",
+"w.w.w.x ;.j s s 8 7 v [ _ [ # w.",
+"w.n x K @.s r 7 6 6 _ 0 % # o w.",
+"w.x *.-.S O.O.X. .d [ % w.w.w.w.",
+"w.y @.r O.z > ; q ~ ~ o w.w.w.w.",
+"w.t O.o. ., w.w.& ~ P o w.w.w.w.",
+"w.4 2 < , - w.w.$ O o   w.w.w.w.",
+"w.w.w.w.w.w.w.w.w.w.w.w.w.w.w.w."
+};
+
+
 /* XPM */
 static char *arrow_rotate_clockwise[] = {
 /* columns rows colors chars-per-pixel */
@@ -3986,6 +4580,16 @@ GLOBALS->hiericon_begin_pixmap=gdk_pixmap_create_from_xpm_d(window->window, &GLO
 GLOBALS->hiericon_fork_pixmap=gdk_pixmap_create_from_xpm_d(window->window, &GLOBALS->hiericon_fork_mask,
 	&style->bg[GTK_STATE_NORMAL], (gchar **)arrow_divide);
 
+/* SV */
+GLOBALS->hiericon_interface_pixmap=gdk_pixmap_create_from_xpm_d(window->window, &GLOBALS->hiericon_interface_mask,
+	&style->bg[GTK_STATE_NORMAL], (gchar **)arrow_inout);
+GLOBALS->hiericon_svpackage_pixmap=gdk_pixmap_create_from_xpm_d(window->window, &GLOBALS->hiericon_svpackage_mask,
+	&style->bg[GTK_STATE_NORMAL], (gchar **)package);
+GLOBALS->hiericon_program_pixmap=gdk_pixmap_create_from_xpm_d(window->window, &GLOBALS->hiericon_program_mask,
+	&style->bg[GTK_STATE_NORMAL], (gchar **)application);
+GLOBALS->hiericon_class_pixmap=gdk_pixmap_create_from_xpm_d(window->window, &GLOBALS->hiericon_class_mask,
+	&style->bg[GTK_STATE_NORMAL], (gchar **)plugin);
+
 /* VHDL */
 GLOBALS->hiericon_design_pixmap=gdk_pixmap_create_from_xpm_d(window->window, &GLOBALS->hiericon_design_mask,
 	&style->bg[GTK_STATE_NORMAL], (gchar **)database);
