@@ -729,6 +729,16 @@ if(GLOBALS->splash_fix_win_title)
 	wave_gtk_window_set_title(GTK_WINDOW(GLOBALS->mainwindow), GLOBALS->winname, GLOBALS->dumpfile_is_modified ? WAVE_SET_TITLE_MODIFIED: WAVE_SET_TITLE_NONE, 0);
 	}
 
+if(GLOBALS->window_entry_c_1)
+        {
+	GLOBALS->entry_raise_timer++;
+	if(GLOBALS->entry_raise_timer > 50)
+		{
+		gdk_window_raise(GLOBALS->window_entry_c_1->window);
+		GLOBALS->entry_raise_timer = 0;
+		}
+        }
+
 #ifdef WAVE_USE_GTK2
 #ifdef MAC_INTEGRATION
 if(GLOBALS->dnd_helper_quartz)
@@ -1240,12 +1250,11 @@ if(GLOBALS->signalpixmap)
 	}
 
  if (!GLOBALS->left_justify_sigs && !GLOBALS->do_resize_signals)
+   {
    if (width < GLOBALS->max_signal_name_pixel_width+15)
      {
        int delta = GLOBALS->max_signal_name_pixel_width+15 - width;
        
-	 GtkAdjustment *hadj;
-
 	 if(GLOBALS->signalpixmap)
 	   {
 
@@ -1264,6 +1273,7 @@ if(GLOBALS->signalpixmap)
        } else {
 	 GLOBALS->right_align_active = 1;
        }
+   }
 
 GLOBALS->old_signal_fill_height= widget->allocation.height;
 gdk_draw_rectangle(GLOBALS->signalpixmap, widget->style->bg_gc[GTK_STATE_PRELIGHT], TRUE, 0, 0,
