@@ -1096,6 +1096,12 @@ for(;;)
                                         case 't':       ttype = TREE_VCD_ST_TASK; break;
                                         case 'f':       ttype = (GLOBALS->yytext_vcd_partial_c_2[1] == 'u') ? TREE_VCD_ST_FUNCTION : TREE_VCD_ST_FORK; break;
                                         case 'b':       ttype = TREE_VCD_ST_BEGIN; break;
+					case 'g':       ttype = TREE_VCD_ST_GENERATE; break;
+					case 's':       ttype = TREE_VCD_ST_STRUCT; break;
+					case 'u':       ttype = TREE_VCD_ST_UNION; break;
+					case 'c':       ttype = TREE_VCD_ST_CLASS; break;
+					case 'i':       ttype = TREE_VCD_ST_INTERFACE; break;
+					case 'p':       ttype = (GLOBALS->yytext_vcd_partial_c_2[1] == 'r') ? TREE_VCD_ST_PROGRAM : TREE_VCD_ST_PACKAGE; break;
                                         default:        ttype = TREE_UNKNOWN;
                                                         break;
                                         }
@@ -2281,15 +2287,26 @@ if(!GLOBALS->hier_was_explicitly_set) /* set default hierarchy split char */
 	GLOBALS->hier_delimeter='.';
 	}
 
+#ifdef __MINGW32__
+if(GetEnvironmentVariable("SHMID", NULL, 0))
+ 	{
+	char sEnv[128];
+	GetEnvironmentVariable("SHMID", sEnv, 128);
+  	sscanf(sEnv, "%x", &shmidu);
+ 	}
+else
+#endif
+	{
+	if(!strcmp(fname, "-vcd"))
+		{
+		if(!fscanf(stdin, "%x", &shmidu)) shmidu = ~(0L); /* allow use of -v flag to pass straight from stdin */
+		}
+		else
+		{
+		sscanf(fname, "%x", &shmidu);	/* passed as a filename */
+		}
+	}
 
-if(!strcmp(fname, "-vcd"))
-	{
-	if(!fscanf(stdin, "%x", &shmidu)) shmidu = ~(0L); /* allow use of -v flag to pass straight from stdin */
-	}
-	else
-	{
-	sscanf(fname, "%x", &shmidu);	/* passed as a filename */
-	}
 shmid = (int)shmidu;
 
 #if !defined _MSC_VER
@@ -2417,7 +2434,7 @@ if(!nd->harray)         /* make quick array lookup for aet display */
 }
 
 /* mark vectors that need to be regenerated */
-static void regen_trace_mark(Trptr t)
+static void regen_trace_mark(Trptr t, int mandclear)
 {
 if(t->vector)
 	{
@@ -2425,6 +2442,21 @@ if(t->vector)
 	bptr bts = b->bits;
 	int i;
 
+	if(1)
+		{
+		for(i=0;i<bts->nnbits;i++)
+			{
+			if(!bts->nodes[i]->expansion)
+				{
+				if(bts->nodes[i]->harray)
+					{
+					free_2(bts->nodes[i]->harray);
+					bts->nodes[i]->harray = NULL;
+					}
+				}
+			}
+		}
+
 	for(i=0;i<bts->nnbits;i++)
 		{
 		if(!bts->nodes[i]->harray)
@@ -2434,6 +2466,15 @@ if(t->vector)
 			}		
 		}
 	}
+	else
+	{
+	if(t->n.nd) /* comment and blank traces don't have a valid node */
+	if((t->n.nd->harray) && (mandclear))
+		{
+		free_2(t->n.nd->harray);
+		t->n.nd->harray = NULL;
+		}
+	}
 }
 
 /* sweep through and regen nodes/dirty vectors */
@@ -2456,13 +2497,22 @@ else if(t->interactive_vector_needs_regeneration)
 
 	for(i=0;i<bts->nnbits;i++)
 		{
+		if(bts->nodes[i]->expansion)
+			{
+			nptr parent = bts->nodes[i]->expansion->parent;
+			int parentbit = bts->nodes[i]->expansion->parentbit;
+
+			DeleteNode(bts->nodes[i]);		
+
+			bts->nodes[i] = ExtractNodeSingleBit(parent, parentbit);
+			}
+
 		if(!bts->nodes[i]->harray)
 			{
 			regen_harray(t, bts->nodes[i]);
 			}		
 		}
 
-
 	if(!bts->name)
 		{
 		bts->name = "";
@@ -2509,7 +2559,6 @@ if(GLOBALS->partial_vcd)
 
 	while(*GLOBALS->consume_ptr_vcd_partial_c_1)
 		{
-		Trptr t;
 		int old_maxtime_marker_conflict = (GLOBALS->tims.marker > GLOBALS->max_time);
 
 		vcd_parse();
@@ -2534,11 +2583,7 @@ if(GLOBALS->partial_vcd)
 			old_maxtime_marker_conflict = (GLOBALS->tims.marker<=GLOBALS->max_time); /* data is now past what was invisible marker */
 			}
 	
-		t = GLOBALS->traces.first; while(t) { regen_trace_mark(t); t = t->t_next; }
-		t = GLOBALS->traces.buffer; while(t) { regen_trace_mark(t); t = t->t_next; }
-
-		t = GLOBALS->traces.first; while(t) { regen_trace_sweep(t); t = t->t_next; }
-		t = GLOBALS->traces.buffer; while(t) { regen_trace_sweep(t); t = t->t_next; }
+		vcd_partial_mark_and_sweep(1);
 	
 		if ((GLOBALS->zoom_dyn) && (!GLOBALS->helpbox_is_active)) 
 			{
@@ -2582,3 +2627,25 @@ if(GLOBALS->partial_vcd)
 gtkwave_main_iteration();
 }
 
+void vcd_partial_mark_and_sweep(int mandclear)
+{
+Trptr t;
+
+/* node */
+t = GLOBALS->traces.first; while(t) { if(!t->vector) regen_trace_mark(t, mandclear); t = t->t_next; }
+t = GLOBALS->traces.buffer; while(t) { if(!t->vector) regen_trace_mark(t, mandclear); t = t->t_next; }
+
+t = GLOBALS->traces.first; while(t) { if(!t->vector) regen_trace_sweep(t); t = t->t_next; }
+t = GLOBALS->traces.buffer; while(t) { if(!t->vector) regen_trace_sweep(t); t = t->t_next; }
+
+/* vector */
+t = GLOBALS->traces.first; while(t) { if(t->vector) regen_trace_mark(t, mandclear); t = t->t_next; }
+t = GLOBALS->traces.buffer; while(t) { if(t->vector) regen_trace_mark(t, mandclear); t = t->t_next; }
+
+t = GLOBALS->traces.first; while(t) { if(t->vector) regen_trace_sweep(t); t = t->t_next; }
+t = GLOBALS->traces.buffer; while(t) { if(t->vector) regen_trace_sweep(t); t = t->t_next; }
+
+/* floating point */
+t = GLOBALS->traces.first; while(t) { if(t->minmax_valid) t->minmax_valid = 0; t = t->t_next; }
+t = GLOBALS->traces.buffer; while(t) { if(t->minmax_valid) t->minmax_valid = 0; t = t->t_next; }
+}
