@@ -1486,6 +1486,12 @@ for(;;)
 					case 't':	ttype = TREE_VCD_ST_TASK; break;
 					case 'f':	ttype = (GLOBALS->yytext_vcd_recoder_c_3[1] == 'u') ? TREE_VCD_ST_FUNCTION : TREE_VCD_ST_FORK; break;
 					case 'b':	ttype = TREE_VCD_ST_BEGIN; break;
+					case 'g':       ttype = TREE_VCD_ST_GENERATE; break;
+					case 's':       ttype = TREE_VCD_ST_STRUCT; break;
+					case 'u':       ttype = TREE_VCD_ST_UNION; break;
+					case 'c':       ttype = TREE_VCD_ST_CLASS; break;
+					case 'i':       ttype = TREE_VCD_ST_INTERFACE; break;
+					case 'p':       ttype = (GLOBALS->yytext_vcd_recoder_c_3[1] == 'r') ? TREE_VCD_ST_PROGRAM : TREE_VCD_ST_PACKAGE; break;
 					default:	ttype = TREE_UNKNOWN;
 							break;
 					}
